import django_filters
import netaddr
from django.db.models import Q
from netaddr.core import AddrFormatError
from netbox.filtersets import NetBoxModelFilterSet
from tenancy.filtersets import TenancyFilterSet

from .models import (
    Community, BGPSession, RoutingPolicy, RoutingPolicyRule,
    BGPPeerGroup, PrefixList, PrefixListRule, CommunityList,
    CommunityListRule, ASPathList, ASPathListRule
)
from ipam.models import IPAddress, ASN
from dcim.models import Device, Site
from virtualization.models import VirtualMachine


class ASPathListFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = ASPathList
        fields = ['id', 'name', 'description']

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(name__icontains=value)
                | Q(description__icontains=value)
        )
        return queryset.filter(qs_filter)


class ASPathListRuleFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = ASPathListRule
        fields = ['id', 'action', 'aspath_list', 'aspath_list_id']

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(action__icontains=value)
                | Q(aspath_list__icontains=value)
                | Q(aspath_list_id__icontains=value)
        )
        return queryset.filter(qs_filter)
        

class CommunityFilterSet(NetBoxModelFilterSet, TenancyFilterSet):

    class Meta:
        model = Community
        fields = ('id', 'value', 'description', 'status', 'tenant',)

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(value__icontains=value)
                | Q(description__icontains=value)
        )
        return queryset.filter(qs_filter)


class CommunityListFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = CommunityList
        fields = ('id', 'name', 'description',)

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(name__icontains=value)
                | Q(description__icontains=value)
        )
        return queryset.filter(qs_filter)


class CommunityListRuleFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = CommunityListRule
        fields = ('id', 'action', 'community_list', 'community_list_id',)

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(action__icontains=value)
                | Q(community_list__icontains=value)
                | Q(community_list_id__icontains=value)
        )
        return queryset.filter(qs_filter)


class BGPSessionFilterSet(NetBoxModelFilterSet, TenancyFilterSet):

    remote_as = django_filters.ModelMultipleChoiceFilter(
        field_name='remote_as__asn',
        queryset=ASN.objects.all(),
        to_field_name='asn',
        label='Remote AS (Number)',
    )
    remote_as_id = django_filters.ModelMultipleChoiceFilter(
        field_name='remote_as__id',
        queryset=ASN.objects.all(),
        to_field_name='id',
        label='Remote AS (ID)',
    )
    local_as = django_filters.ModelMultipleChoiceFilter(
        field_name='local_as__asn',
        queryset=ASN.objects.all(),
        to_field_name='asn',
        label='Local AS (Number)',
    )
    local_as_id = django_filters.ModelMultipleChoiceFilter(
        field_name='local_as__id',
        queryset=ASN.objects.all(),
        to_field_name='id',
        label='Local AS (ID)',
    )
    peer_group = django_filters.ModelMultipleChoiceFilter(
        queryset=BGPPeerGroup.objects.all(),
    )
    import_policies = django_filters.ModelMultipleChoiceFilter(
        queryset=RoutingPolicy.objects.all(),
    )
    export_policies = django_filters.ModelMultipleChoiceFilter(
        queryset=RoutingPolicy.objects.all(),
    )
    local_address_id = django_filters.ModelMultipleChoiceFilter(
        field_name='local_address__id',
        queryset=IPAddress.objects.all(),
        to_field_name='id',
        label='Local Address (ID)',
    )
    local_address = django_filters.ModelMultipleChoiceFilter(
        field_name='local_address__address',
        queryset=IPAddress.objects.all(),
        to_field_name='address',
        label='Local Address',
    )
    remote_address_id = django_filters.ModelMultipleChoiceFilter(
        field_name='remote_address__id',
        queryset=IPAddress.objects.all(),
        to_field_name='id',
        label='Remote Address (ID)',
    )
    remote_address = django_filters.ModelMultipleChoiceFilter(
        field_name='remote_address__address',
        queryset=IPAddress.objects.all(),
        to_field_name='address',
        label='Remote Address',
    )
    device_id = django_filters.ModelMultipleChoiceFilter(
        field_name='device__id',
        queryset=Device.objects.all(),
        to_field_name='id',
        label='Device (ID)',
    )
    device = django_filters.ModelMultipleChoiceFilter(
        field_name='device__name',
        queryset=Device.objects.all(),
        to_field_name='name',
        label='Device (name)',
    )
    virtualmachine_id = django_filters.ModelMultipleChoiceFilter(
        field_name='virtualmachine__id',
        queryset=VirtualMachine.objects.all(),
        to_field_name='id',
        label='VirtualMachine (ID)',
    )
    virtualmachine = django_filters.ModelMultipleChoiceFilter(
        field_name='virtualmachine__name',
        queryset=VirtualMachine.objects.all(),
        to_field_name='name',
        label='VirtualMachine (name)',
    )
    site_id = django_filters.ModelMultipleChoiceFilter(
        field_name='site__id',
        queryset=Site.objects.all(),
        to_field_name='id',
        label='Site (ID)',
    )
    site = django_filters.ModelMultipleChoiceFilter(
        field_name='site__name',
        queryset=Site.objects.all(),
        to_field_name='name',
        label='DSite (name)',
    )    
    by_remote_address = django_filters.CharFilter(
        method='search_by_remote_ip',
        label='Remote Address',
    )
    by_local_address = django_filters.CharFilter(
        method='search_by_local_ip',
        label='Local Address',
    )

    class Meta:
        model = BGPSession
        fields = ('id', 'name', 'description', 'status', 'tenant',)

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(remote_as__asn__icontains=value)
                | Q(name__icontains=value)
                | Q(local_as__asn__icontains=value)
                | Q(description__icontains=value)
        )
        return queryset.filter(qs_filter)

    def search_by_remote_ip(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = str(netaddr.IPNetwork(value).cidr)
            return queryset.filter(remote_address__address=query)
        except (AddrFormatError, ValueError):
            return queryset.none()

    def search_by_local_ip(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = str(netaddr.IPNetwork(value).cidr)
            return queryset.filter(local_address__address=query)
        except (AddrFormatError, ValueError):
            return queryset.none()


class RoutingPolicyFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = RoutingPolicy
        fields = ('id', 'name', 'description',)

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(name__icontains=value)
                | Q(description__icontains=value)
        )
        return queryset.filter(qs_filter)


class RoutingPolicyRuleFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = RoutingPolicyRule
        fields = ('id', 'index', 'action', 'description', 'routing_policy_id', 'continue_entry',)

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(index__icontains=value)
                | Q(action__icontains=value)
                | Q(description__icontains=value)
                | Q(routing_policy_id__icontains=value)
                | Q(continue_entry__icontains=value)
        )
        return queryset.filter(qs_filter)


class BGPPeerGroupFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = BGPPeerGroup
        fields = ('id', 'name', 'description',)

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(name__icontains=value)
                | Q(description__icontains=value)
        )
        return queryset.filter(qs_filter)


class PrefixListFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = PrefixList
        fields = ('id', 'name', 'description', 'family',)

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(name__icontains=value)
                | Q(description__icontains=value)
        )
        return queryset.filter(qs_filter)

class PrefixListRuleFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = PrefixListRule
        #fields = ['index', 'action', 'prefix_custom', 'ge', 'le', 'prefix_list', 'prefix_list_id']
        fields = ('id', 'index', 'action', 'ge', 'le', 'prefix_list', 'prefix_list_id',)

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(index__icontains=value)
                | Q(action__icontains=value)
                #| Q(prefix_custom__icontains=value)
                | Q(ge__icontains=value)
                | Q(le__icontains=value)
                | Q(prefix_list__icontains=value)
                | Q(prefix_list_id__icontains=value)
        )
        return queryset.filter(qs_filter)



# Generated by Django 4.2.8 on 2024-02-16 10:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0031_netbox_bgp'),
    ]

    operations = [
        migrations.AddField(
            model_name='bgpsession',
            name='prefix_list_in',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='session_prefix_in', to='netbox_bgp.prefixlist'),
        ),
        migrations.AddField(
            model_name='bgpsession',
            name='prefix_list_out',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='session_prefix_out', to='netbox_bgp.prefixlist'),
        ),
    ]



from typing import List

import strawberry
import strawberry_django

from netbox_bgp.models import (
    Community,
    BGPSession,
    RoutingPolicy,
    BGPPeerGroup,
    RoutingPolicyRule,
    PrefixList,
    PrefixListRule,
    CommunityList,
    CommunityListRule,
    ASPathList,
    ASPathListRule
)
from .types import (
    CommunityType,
    BGPSessionType,
    BGPPeerGroupType,
    RoutingPolicyType,
    RoutingPolicyRuleType,
    PrefixListType,
    PrefixListRuleType,
    CommunityListType,
    CommunityListRuleType,
    ASPathListType,
    ASPathListRuleType
)


@strawberry.type(name="Query")
class NetBoxBGPQuery:

    netbox_bgp_community: CommunityType = strawberry_django.field()
    netbox_bgp_community_list: List[CommunityType] = strawberry_django.field()

    netbox_bgp_session: BGPSessionType = strawberry_django.field()
    netbox_bgp_session_list: List[BGPSessionType] = strawberry_django.field()

    netbox_bgp_peer_group: BGPPeerGroupType = strawberry_django.field()
    netbox_bgp_peer_group_list: List[BGPPeerGroupType] = strawberry_django.field()

    netbox_bgp_routing_policy: RoutingPolicyType = strawberry_django.field()
    netbox_bgp_routing_policy_list: List[RoutingPolicyType] = strawberry_django.field()

    netbox_bgp_routing_policy_rule: RoutingPolicyRuleType = strawberry_django.field()
    netbox_bgp_routing_policy_rule_list: List[RoutingPolicyRuleType] = strawberry_django.field()

    netbox_bgp_prefixlist: PrefixListType = strawberry_django.field()
    netbox_bgp_prefixlist_list: List[PrefixListType] = strawberry_django.field()

    netbox_bgp_prefixlist_rule: PrefixListRuleType = strawberry_django.field()
    netbox_bgp_prefixlist_rule_list: List[PrefixListRuleType] = strawberry_django.field()

    netbox_bgp_communitylist: CommunityListType = strawberry_django.field()
    netbox_bgp_communitylist_list: List[CommunityListType] = strawberry_django.field()

    netbox_bgp_communitylist_rule: CommunityListRuleType = strawberry_django.field()
    netbox_bgp_communitylist_rule_list: List[CommunityListRuleType] = strawberry_django.field()

    netbox_bgp_aspathlist: ASPathListType = strawberry_django.field()
    netbox_bgp_aspathlist_list: List[ASPathListType] = strawberry_django.field()

    netbox_bgp_aspathlist_rule: ASPathListRuleType = strawberry_django.field()
    netbox_bgp_aspathlist_rule_list: List[ASPathListRuleType] = strawberry_django.field()


from django import forms
from utilities.forms.rendering import FieldSet
from django.core.exceptions import (
    MultipleObjectsReturned,
    ObjectDoesNotExist,
    ValidationError,
)
from django.utils.translation import gettext as _

from tenancy.models import Tenant
from dcim.models import Device, Site
from ipam.models import IPAddress, Prefix, ASN
from ipam.formfields import IPNetworkFormField
from utilities.forms.fields import (
    DynamicModelChoiceField,
    CSVModelChoiceField,
    CSVModelMultipleChoiceField,
    DynamicModelMultipleChoiceField,
    TagFilterField,
    CSVChoiceField,
    CommentField,
)
from utilities.forms import add_blank_choice
from utilities.forms.widgets import APISelect, APISelectMultiple
from netbox.forms import (
    NetBoxModelForm,
    NetBoxModelBulkEditForm,
    NetBoxModelFilterSetForm,
    NetBoxModelImportForm,
)
from .choices import SessionStatusChoices

from .models import (
    Community,
    BGPSession,
    RoutingPolicy,
    BGPPeerGroup,
    RoutingPolicyRule,
    PrefixList,
    PrefixListRule,
    CommunityList,
    CommunityListRule,
    ASPathList,
    ASPathListRule,
)

from .choices import (
    SessionStatusChoices,
    CommunityStatusChoices,
    IPAddressFamilyChoices,
)

from virtualization.models import VirtualMachine

class ASPathListFilterForm(NetBoxModelFilterSetForm):
    model = ASPathList
    q = forms.CharField(required=False, label="Search")

    tag = TagFilterField(model)

class ASPathListRuleFilterForm(NetBoxModelFilterSetForm):
    model = ASPathListRule
    q = forms.CharField(required=False, label="Search")
    aspath_list = DynamicModelChoiceField(queryset=ASPathList.objects.all(), required=False)
    tag = TagFilterField(model)

    
class ASPathListForm(NetBoxModelForm):

    comments = CommentField()

    class Meta:
        model = ASPathList
        fields = ["name", "description", "tags", "comments"]


class ASPathListBulkEditForm(NetBoxModelBulkEditForm):
    description = forms.CharField(max_length=200, required=False)

    model = ASPathList
    nullable_fields = [
        "description",
    ]


class ASPathListImportForm(NetBoxModelImportForm):

    class Meta:
        model = ASPathList
        fields = ["name", "description", "tags"]


class ASPathListRuleImportForm(NetBoxModelImportForm):

    class Meta:
        model = ASPathListRule
        fields = ["aspath_list", "index", "action", "pattern", "description", "tags", "comments"]   


class ASPathListRuleForm(NetBoxModelForm):
    comments = CommentField()

    class Meta:
        model = ASPathListRule
        fields = ["aspath_list", "index", "action", "pattern", "description", "tags", "comments"]


class CommunityForm(NetBoxModelForm):
    status = forms.ChoiceField(
        required=False,
        choices=CommunityStatusChoices,
    )
    tenant = DynamicModelChoiceField(queryset=Tenant.objects.all(), required=False)
    comments = CommentField()

    class Meta:
        model = Community
        fields = ["value", "description", "status", "tenant", "tags", "comments"]


class CommunityFilterForm(NetBoxModelFilterSetForm):
    q = forms.CharField(required=False, label="Search")
    tenant = DynamicModelChoiceField(queryset=Tenant.objects.all(), required=False)
    status = forms.MultipleChoiceField(
        choices=CommunityStatusChoices,
        required=False,
    )
    site = DynamicModelChoiceField(queryset=Site.objects.all(), required=False)

    tag = TagFilterField(Community)

    model = Community


class CommunityBulkEditForm(NetBoxModelBulkEditForm):
    tenant = DynamicModelChoiceField(queryset=Tenant.objects.all(), required=False)
    description = forms.CharField(max_length=200, required=False)
    status = forms.ChoiceField(
        required=False,
        choices=CommunityStatusChoices,
    )

    model = Community
    nullable_fields = [
        "tenant",
        "description",
    ]


class CommunityImportForm(NetBoxModelImportForm):
    tenant = CSVModelChoiceField(
        queryset=Tenant.objects.all(),
        required=False,
        to_field_name="name",
        help_text=_("Assigned tenant"),
    )

    status = CSVChoiceField(
        choices=CommunityStatusChoices, help_text=_("Operational status")
    )

    class Meta:
        model = Community
        fields = ("value", "description", "tags")


class CommunityListFilterForm(NetBoxModelFilterSetForm):
    model = CommunityList
    q = forms.CharField(required=False, label="Search")

    tag = TagFilterField(model)


class CommunityListForm(NetBoxModelForm):

    comments = CommentField()

    class Meta:
        model = CommunityList
        fields = ["name", "description", "tags", "comments"]


class CommunityListBulkEditForm(NetBoxModelBulkEditForm):
    description = forms.CharField(max_length=200, required=False)

    model = CommunityList
    nullable_fields = [
        "description",
    ]


class CommunityListImportForm(NetBoxModelImportForm):

    class Meta:
        model = CommunityList
        fields = ("name", "description", "tags")


class CommunityListRuleForm(NetBoxModelForm):
    community = DynamicModelChoiceField(
        queryset=Community.objects.all(),
        required=False,
        help_text="Community",
    )

    comments = CommentField()

    class Meta:
        model = CommunityListRule
        fields = ["community_list", "action", "community", "tags", "comments"]


class BGPSessionForm(NetBoxModelForm):
    name = forms.CharField(max_length=64, required=True)
    site = DynamicModelChoiceField(queryset=Site.objects.all(), required=False)
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(), required=False, query_params={"site_id": "$site"}
    )
    virtualmachine = DynamicModelChoiceField(
        queryset=VirtualMachine.objects.all(), required=False, query_params={"site_id": "$site"}
    )

    tenant = DynamicModelChoiceField(queryset=Tenant.objects.all(), required=False)
    local_as = DynamicModelChoiceField(
        queryset=ASN.objects.all(),
        query_params={"site_id": "$site"},
        label=_("Local AS"),
    )
    remote_as = DynamicModelChoiceField(
        queryset=ASN.objects.all(), label=_("Remote AS")
    )
    local_address = DynamicModelChoiceField(
        queryset=IPAddress.objects.all(), query_params={"device_id": "$device"}
    )
    remote_address = DynamicModelChoiceField(
        queryset=IPAddress.objects.all(),
    )
    peer_group = DynamicModelChoiceField(
        queryset=BGPPeerGroup.objects.all(),
        required=False,
        widget=APISelect(
            api_url="/api/plugins/bgp/peer-group/",
        ),
    )
    import_policies = DynamicModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/routing-policy/"),
    )
    export_policies = DynamicModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/routing-policy/"),
    )
    prefix_list_in = DynamicModelChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        widget=APISelect(
            api_url="/api/plugins/bgp/prefix-list/",
        ),
    )
    prefix_list_out = DynamicModelChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        widget=APISelect(
            api_url="/api/plugins/bgp/prefix-list/",
        ),
    )
    comments = CommentField()


    fieldsets = (
        FieldSet(
            "name",
            "description",
            "site",
            "device",
            "virtualmachine",
            "status",
            "peer_group",
            "tenant",
            "tags",
            name="Session",
        ),
        FieldSet("remote_as", "remote_address", name="Remote"),
        FieldSet("local_as", "local_address", name="Local"),
        FieldSet("import_policies", "export_policies", name="Policies"),
        FieldSet("prefix_list_in", "prefix_list_out", name="Prefixes"),
    )

    class Meta:
        model = BGPSession
        fields = [
            "name",
            "site",
            "device",
            "virtualmachine",
            "local_as",
            "remote_as",
            "local_address",
            "remote_address",
            "description",
            "status",
            "peer_group",
            "tenant",
            "tags",
            "import_policies",
            "export_policies",
            "prefix_list_in",
            "prefix_list_out",
            "comments",
        ]

        widgets = {
            "status": forms.Select(),
        }


class BGPSessionAddForm(BGPSessionForm):
    remote_address = IPNetworkFormField()

    def clean_remote_address(self):
        try:
            ip = IPAddress.objects.get(address=str(self.cleaned_data["remote_address"]))
        except MultipleObjectsReturned:
            ip = IPAddress.objects.filter(
                address=str(self.cleaned_data["remote_address"])
            ).first()
        except ObjectDoesNotExist:
            ip = IPAddress.objects.create(
                address=str(self.cleaned_data["remote_address"])
            )
        self.cleaned_data["remote_address"] = ip
        return self.cleaned_data["remote_address"]


class BGPSessionImportForm(NetBoxModelImportForm):
    site = CSVModelChoiceField(
        label=_("Site"),
        required=False,
        queryset=Site.objects.all(),
        to_field_name="name",
        help_text=_("Assigned site"),
    )
    tenant = CSVModelChoiceField(
        queryset=Tenant.objects.all(),
        required=False,
        to_field_name="name",
        help_text=_("Assigned tenant"),
    )
    device = CSVModelChoiceField(
        queryset=Device.objects.all(),
        to_field_name="name",
        help_text=_("Assigned device"),
    )
    virtualmachine = CSVModelChoiceField(
        queryset=VirtualMachine.objects.all(),
        to_field_name="name",
        help_text=_("Assigned virtual machine"),
    )
    status = CSVChoiceField(
        choices=SessionStatusChoices, required=False, help_text=_("Operational status")
    )
    local_address = CSVModelChoiceField(
        queryset=IPAddress.objects.all(),
        to_field_name="address",
        help_text=_("Local IP Address"),
    )
    remote_address = CSVModelChoiceField(
        queryset=IPAddress.objects.all(),
        to_field_name="address",
        help_text=_("Remote IP Address"),
    )
    local_as = CSVModelChoiceField(
        queryset=ASN.objects.all(),
        to_field_name="asn",
        help_text=_("Local ASN"),
    )
    remote_as = CSVModelChoiceField(
        queryset=ASN.objects.all(),
        to_field_name="asn",
        help_text=_("Remote ASN"),
    )
    peer_group = CSVModelChoiceField(
        queryset=BGPPeerGroup.objects.all(),
        required=False,
        to_field_name="name",
        help_text=_("Peer Group"),
    )
    import_policies = CSVModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        to_field_name="name",
        required=False,
        help_text=_("Import policies name"),
    )
    export_policies = CSVModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        to_field_name="name",
        required=False,
        help_text=_("Export policies name"),
    )
    prefix_list_in = CSVModelChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        to_field_name="name",
        help_text=_("Prefix list In"),
    )
    prefix_list_out = CSVModelChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        to_field_name="name",
        help_text=_("Prefix List Out"),
    )

    class Meta:
        model = BGPSession
        fields = [
            "name",
            "device",
            "virtualmachine",
            "site",
            "description",
            "tenant",
            "status",
            "peer_group",
            "import_policies",
            "export_policies",
            "local_address",
            "remote_address",
            "local_as",
            "remote_as",
            "tags",
            "prefix_list_in",
            "prefix_list_out",
        ]


class BGPSessionFilterForm(NetBoxModelFilterSetForm):
    model = BGPSession
    q = forms.CharField(required=False, label="Search")
    remote_as_id = DynamicModelMultipleChoiceField(
        queryset=ASN.objects.all(), required=False, label=_("Remote AS")
    )
    local_as_id = DynamicModelMultipleChoiceField(
        queryset=ASN.objects.all(), required=False, label=_("Local AS")
    )
    by_local_address = forms.CharField(required=False, label="Local Address")
    by_remote_address = forms.CharField(required=False, label="Remote Address")
    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(), required=False, label=_("Device")
    )
    virtualmachine_id = DynamicModelMultipleChoiceField(
        queryset=VirtualMachine.objects.all(), required=False, label=_("VirtualMachine")
    )
    site_id = DynamicModelMultipleChoiceField(
        queryset=Site.objects.all(), required=False, label=_("Site")
    )
    status = forms.MultipleChoiceField(
        choices=SessionStatusChoices,
        required=False,
    )
    peer_group = DynamicModelMultipleChoiceField(
        queryset=BGPPeerGroup.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/peer-group/"),
    )
    import_policies = DynamicModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/routing-policy/"),
    )
    export_policies = DynamicModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/routing-policy/"),
    )
    prefix_list_in = DynamicModelMultipleChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/prefix-list/"),
    )
    prefix_list_out = DynamicModelMultipleChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/prefix-list/"),
    )
    tenant = DynamicModelChoiceField(queryset=Tenant.objects.all(), required=False)

    tag = TagFilterField(model)


class BGPSessionBulkEditForm(NetBoxModelBulkEditForm):
    device = DynamicModelChoiceField(
        label=_("Device"),
        queryset=Device.objects.all(),
        required=False,
    )
    virtualmachine = DynamicModelChoiceField(
        label=_("Virtual Machine"),
        queryset=VirtualMachine.objects.all(),
        required=False,
    )
    site = DynamicModelChoiceField(
        label=_("Site"), queryset=Site.objects.all(), required=False
    )

    status = forms.ChoiceField(
        label=_('Status'),
        choices=add_blank_choice(SessionStatusChoices),
        required=False
    )
    description = forms.CharField(
        label=_("Description"), max_length=200, required=False
    )
    tenant = DynamicModelChoiceField(
        label=_("Tenant"), queryset=Tenant.objects.all(), required=False
    )
    local_as = DynamicModelChoiceField(queryset=ASN.objects.all(), required=False)
    remote_as = DynamicModelChoiceField(queryset=ASN.objects.all(), required=False)
    peer_group = DynamicModelChoiceField(
        queryset=BGPPeerGroup.objects.all(),
        required=False,
        widget=APISelect(
            api_url="/api/plugins/bgp/peer-group/",
        ),
    )
    import_policies = DynamicModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/routing-policy/"),
    )
    export_policies = DynamicModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/routing-policy/"),
    )

    model = BGPSession

    fieldsets = (
        FieldSet(
            "name",
            "description",
            "site",
            "device",
            "virtualmachine",
            "status",
            "peer_group",
            "tenant",
            "tags",
            name="Session",
        ),
        FieldSet("remote_as", "remote_address", name="Remote"),
        FieldSet("local_as", "local_address", name="Local"),
        FieldSet("import_policies", "export_policies", name="Policies"),
        FieldSet("prefix_list_in", "prefix_list_out", name="Prefixes"),
    )

    nullable_fields = [
        "tenant",
        "description",
        "peer_group",
        "import_policies",
        "export_policies",
        "prefix_list_in",
        "prefix_list_out",
    ]


class RoutingPolicyFilterForm(NetBoxModelFilterSetForm):
    model = RoutingPolicy
    q = forms.CharField(required=False, label="Search")

    tag = TagFilterField(model)


class RoutingPolicyForm(NetBoxModelForm):

    comments = CommentField()

    class Meta:
        model = RoutingPolicy
        fields = ["name", "description", "tags", "comments"]


class RoutingPolicyImportForm(NetBoxModelImportForm):

    class Meta:
        model = RoutingPolicy
        fields = ("name", "description", "tags")


class RoutingPolicyBulkEditForm(NetBoxModelBulkEditForm):
    description = forms.CharField(max_length=200, required=False)

    model = RoutingPolicy
    nullable_fields = [
        "description",
    ]


class BGPPeerGroupFilterForm(NetBoxModelFilterSetForm):
    model = BGPPeerGroup
    q = forms.CharField(required=False, label="Search")

    tag = TagFilterField(model)


class BGPPeerGroupForm(NetBoxModelForm):
    import_policies = DynamicModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/routing-policy/"),
    )
    export_policies = DynamicModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/routing-policy/"),
    )
    comments = CommentField()

    class Meta:
        model = BGPPeerGroup
        fields = [
            "name",
            "description",
            "import_policies",
            "export_policies",
            "tags",
            "comments",
        ]


class BGPPeerGroupImportForm(NetBoxModelImportForm):

    import_policies = CSVModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        to_field_name="name",
        required=False,
        help_text=_("Import policies name"),
    )
    export_policies = CSVModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        to_field_name="name",
        required=False,
        help_text=_("Export policies name"),
    )

    class Meta:
        model = BGPPeerGroup
        fields = ("name", "description", "import_policies", "export_policies", "tags")


class BGPPeerGroupBulkEditForm(NetBoxModelBulkEditForm):
    description = forms.CharField(max_length=200, required=False)

    import_policies = DynamicModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/routing-policy/"),
    )
    export_policies = DynamicModelMultipleChoiceField(
        queryset=RoutingPolicy.objects.all(),
        required=False,
        widget=APISelectMultiple(api_url="/api/plugins/bgp/routing-policy/"),
    )

    model = BGPPeerGroup
    nullable_fields = [
        "description", "import_policies", "export_policies"
    ]


class RoutingPolicyRuleForm(NetBoxModelForm):
    continue_entry = forms.IntegerField(
        required=False,
        label="Continue",
        help_text="Null for disable, 0 to next entry, or any sequence number",
    )
    match_community = DynamicModelMultipleChoiceField(
        queryset=Community.objects.all(),
        required=False,
    )
    match_community_list = DynamicModelMultipleChoiceField(
        queryset=CommunityList.objects.all(),
        required=False,
    )
    
    match_ip_address = DynamicModelMultipleChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        query_params={
            'family': 'ipv4'
        }
    )

    match_ipv6_address = DynamicModelMultipleChoiceField(
        queryset=PrefixList.objects.all(),
        required=False,
        query_params={
            'family': 'ipv6'
        }
    )

    match_aspath_list = DynamicModelMultipleChoiceField(
        queryset=ASPathList.objects.all(),
        required=False,
    )   

    match_custom = forms.JSONField(
        label="Custom Match",
        help_text='Any custom match statements, e.g., {"ip nexthop": "1.1.1.1"}',
        required=False,
    )
    set_actions = forms.JSONField(
        label="Set statements",
        help_text='Set statements, e.g., {"as-path prepend": [12345,12345]}',
        required=False,
    )
    comments = CommentField()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        instance = kwargs.get("instance", {})

    class Meta:
        model = RoutingPolicyRule
        fields = [
            "routing_policy",
            "index",
            "action",
            "continue_entry",
            "match_community",
            "match_community_list",
            "match_ip_address",
            "match_ipv6_address",
            "match_aspath_list",
            "match_custom",
            "set_actions",
            "description",
            "tags",
            "comments",
        ]

class RoutingPolicyRuleImportForm(NetBoxModelImportForm):

    class Meta:
        model = RoutingPolicyRule
        fields = (
            "routing_policy",
            "index",
            "action",
            "continue_entry",
            "match_community",
            "match_community_list",
            "match_ip_address",
            "match_ipv6_address",
            "match_custom",
            "set_actions",
            "description",
            "tags",
            "comments",
        )



class PrefixListFilterForm(NetBoxModelFilterSetForm):
    model = PrefixList
    q = forms.CharField(required=False, label="Search")

    tag = TagFilterField(model)


class PrefixListForm(NetBoxModelForm):

    comments = CommentField()

    class Meta:
        model = PrefixList
        fields = ["name", "description", "family", "tags", "comments"]


class PrefixListImportForm(NetBoxModelImportForm):
    family = CSVChoiceField(
        choices=IPAddressFamilyChoices, required=True, help_text=_("Family address")
    )

    class Meta:
        model = PrefixList
        fields = ("name", "description", "family", "tags")


class PrefixListBulkEditForm(NetBoxModelBulkEditForm):
    description = forms.CharField(max_length=200, required=False)

    family = forms.ChoiceField(
        label=_("Family"),
        required=False,
        choices=IPAddressFamilyChoices,
    )

    model = PrefixList
    nullable_fields = [
        "description",
    ]

class PrefixListRuleImportForm(NetBoxModelImportForm):

    class Meta:
        model = PrefixListRule
        fields = (
            "prefix_list",
            "index",
            "action",
            "prefix",
            "prefix_custom",
            "ge",
            "le",
            "tags",
            "comments",
        )
        
class PrefixListRuleForm(NetBoxModelForm):
    prefix = DynamicModelChoiceField(
        queryset=Prefix.objects.all(),
        required=False,
        help_text="NetBox Prefix Object",
    )
    prefix_custom = IPNetworkFormField(
        required=False,
        label="Prefix",
        help_text="Just IP field for define special prefix like 0.0.0.0/0",
    )
    ge = forms.IntegerField(
        label="Greater than or equal to",
        required=False,
    )
    le = forms.IntegerField(
        label="Less than or equal to",
        required=False,
    )
    comments = CommentField()

    class Meta:
        model = PrefixListRule
        fields = [
            "prefix_list",
            "index",
            "action",
            "prefix",
            "prefix_custom",
            "ge",
            "le",
            "tags",
            "comments",
        ]



from typing import Annotated, List
import strawberry
import strawberry_django
from netbox.graphql.types import NetBoxObjectType
from netbox.graphql.scalars import BigInt

from netbox_bgp.models import (
    Community,
    BGPSession,
    RoutingPolicy,
    BGPPeerGroup,
    RoutingPolicyRule,
    PrefixList,
    PrefixListRule,
    CommunityList,
    CommunityListRule,
    ASPathList,
    ASPathListRule
)
from .filters import (
    NetBoxBGPCommunityFilter,
    NetBoxBGPSessionFilter,
    NetBoxBGPBGPPeerGroupFilter,
    NetBoxBGPRoutingPolicyFilter,
    NetBoxBGPRoutingPolicyRuleFilter,
    NetBoxBGPPrefixListFilter,
    NetBoxBGPPrefixListRuleFilter,
    NetBoxBGPCommunityListFilter,
    NetBoxBGPCommunityListRuleFilter,
    NetBoxBGPASPathListFilter,
    NetBoxBGPASPathListRuleFilter
)

@strawberry_django.type(ASPathList, fields="__all__", filters=NetBoxBGPASPathListFilter)
class ASPathListType(NetBoxObjectType):
    name: str
    description: str
    rules: List[
         Annotated["ASPathListRuleType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]


@strawberry_django.type(ASPathListRule, fields="__all__", filters=NetBoxBGPASPathListRuleFilter)
class ASPathListRuleType(NetBoxObjectType):
    aspath_list: Annotated[
        "ASPathListType", strawberry.lazy("netbox_bgp.graphql.types")
    ]
    index: BigInt
    action: str
    pattern: str
    description: str


@strawberry_django.type(Community, fields="__all__", filters=NetBoxBGPCommunityFilter)
class CommunityType(NetBoxObjectType):
    site: Annotated["SiteType", strawberry.lazy("dcim.graphql.types")] | None
    tenant: Annotated["TenantType", strawberry.lazy("tenancy.graphql.types")] | None
    status: str
    role: Annotated["RoleType", strawberry.lazy("ipam.graphql.types")] | None
    description: str


@strawberry_django.type(BGPSession, fields="__all__", filters=NetBoxBGPSessionFilter)
class BGPSessionType(NetBoxObjectType):
    name: str
    site: Annotated["SiteType", strawberry.lazy("dcim.graphql.types")] | None
    tenant: Annotated["TenantType", strawberry.lazy("tenancy.graphql.types")] | None
    device: Annotated["DeviceType", strawberry.lazy("dcim.graphql.types")] | None
    virtualmachine: Annotated["VirtualMachineType", strawberry.lazy("virtualization.graphql.types")] | None
    local_address: Annotated["IPAddressType", strawberry.lazy("ipam.graphql.types")]
    remote_address: Annotated["IPAddressType", strawberry.lazy("ipam.graphql.types")]
    local_as: Annotated["ASNType", strawberry.lazy("ipam.graphql.types")]
    remote_as: Annotated["ASNType", strawberry.lazy("ipam.graphql.types")]
    status: str
    description: str
    peer_group: (
        Annotated["BGPPeerGroupType", strawberry.lazy("netbox_bgp.graphql.types")]
        | None
    )
    import_policies: List[
        Annotated["RoutingPolicyType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]
    export_policies: List[
        Annotated["RoutingPolicyType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]
    prefix_list_in: Annotated["PrefixListType", strawberry.lazy("netbox_bgp.graphql.types")] | None
    prefix_list_out: Annotated["PrefixListType", strawberry.lazy("netbox_bgp.graphql.types")] | None


@strawberry_django.type(BGPPeerGroup, fields="__all__", filters=NetBoxBGPBGPPeerGroupFilter)
class BGPPeerGroupType(NetBoxObjectType):
    name: str
    description: str
    import_policies: List[
        Annotated["RoutingPolicyType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]
    export_policies: List[
        Annotated["RoutingPolicyType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]


@strawberry_django.type(RoutingPolicy, fields="__all__", filters=NetBoxBGPRoutingPolicyFilter)
class RoutingPolicyType(NetBoxObjectType):
    name: str
    description: str
    rules: List[
         Annotated["RoutingPolicyRuleType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]


@strawberry_django.type(
    RoutingPolicyRule, fields="__all__", filters=NetBoxBGPRoutingPolicyRuleFilter
)
class RoutingPolicyRuleType(NetBoxObjectType):
    routing_policy: Annotated[
        "RoutingPolicyType", strawberry.lazy("netbox_bgp.graphql.types")
    ]
    index: BigInt
    action: str
    description: str
    continue_entry: BigInt | None
    match_community: List[
        Annotated["CommunityType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]
    match_community_list: List[
        Annotated["CommunityListType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]
    match_ip_address: List[
        Annotated["PrefixListType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]
    match_ipv6_address: List[
        Annotated["PrefixListType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]


@strawberry_django.type(PrefixList, fields="__all__", filters=NetBoxBGPPrefixListFilter)
class PrefixListType(NetBoxObjectType):
    name: str
    description: str
    family: str
    prefrules: List[
         Annotated["PrefixListRuleType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]


@strawberry_django.type(PrefixListRule, fields="__all__", filters=NetBoxBGPPrefixListRuleFilter)
class PrefixListRuleType(NetBoxObjectType):
    prefix_list: Annotated[
        "PrefixListType", strawberry.lazy("netbox_bgp.graphql.types")
    ]
    index: BigInt
    action: str
    prefix: Annotated["PrefixType", strawberry.lazy("ipam.graphql.types")] | None
    prefix_custom: str | None
    ge: BigInt
    le: BigInt
    description: str


@strawberry_django.type(CommunityList, fields="__all__", filters=NetBoxBGPCommunityListFilter)
class CommunityListType(NetBoxObjectType):
    name: str
    description: str
    commlistrules: List[
         Annotated["CommunityListRuleType", strawberry.lazy("netbox_bgp.graphql.types")]
    ]


@strawberry_django.type(
    CommunityListRule, fields="__all__", filters=NetBoxBGPCommunityListRuleFilter
)
class CommunityListRuleType(NetBoxObjectType):
    community_list: Annotated[
        "CommunityListType", strawberry.lazy("netbox_bgp.graphql.types")
    ]
    action: str
    community: Annotated["CommunityType", strawberry.lazy("netbox_bgp.graphql.types")]
    description: str



from django.urls import include, path
from utilities.urls import get_model_urls

from .models import (
    BGPSession, Community, RoutingPolicy,
    BGPPeerGroup, RoutingPolicyRule, PrefixList,
    PrefixListRule, CommunityList, CommunityListRule
)
from . import views

app_name = 'netbox_bgp'

urlpatterns = (
    # AS Path Lists
    path(
        "aspath-list/",
        include(get_model_urls("netbox_bgp", "aspathlist", detail=False)),
    ),
    path(
        "aspath-list/<int:pk>/",
        include(get_model_urls("netbox_bgp", "aspathlist")),
    ),
    # AS Path List Rules
    path(
        "aspath-list-rule/",
        include(get_model_urls("netbox_bgp", "aspathlistrule", detail=False)),
    ),
    path(
        "aspath-list-rule/<int:pk>/",
        include(get_model_urls("netbox_bgp","aspathlistrule")),
    ),
    # Community
    path(
        "community/",
        include(get_model_urls("netbox_bgp", "community", detail=False)),
    ),
    path(
        "community/<int:pk>/",
        include(get_model_urls("netbox_bgp", "community")),
    ),

    # Community Lists
    path(
        "community-list/",
        include(get_model_urls("netbox_bgp", "communitylist", detail=False)),
    ),
    path(
        "community-list/<int:pk>/",
        include(get_model_urls("netbox_bgp", "communitylist")),
    ),

    # Community List Rules
    path(
        "community-list-rule/",
        include(get_model_urls("netbox_bgp", "communitylistrule", detail=False)),
    ),
    path(
        "community-list-rule/<int:pk>/",
        include(get_model_urls("netbox_bgp", "communitylistrule")),
    ),

    # Sessions
    path(
        "session/",
        include(get_model_urls("netbox_bgp", "bgpsession", detail=False)),
    ),
    path(
        "session/<int:pk>/",
        include(get_model_urls("netbox_bgp", "bgpsession")),
    ),

    # Routing Policies
    path(
        "routing-policy/",
        include(get_model_urls("netbox_bgp", "routingpolicy", detail=False)),
    ),
    path(
        "routing-policy/<int:pk>/",
        include(get_model_urls("netbox_bgp", "routingpolicy")),
    ),

    # Routing Policy Rules
    path(
        "routing-policy-rule/",
        include(get_model_urls("netbox_bgp", "routingpolicyrule", detail=False)),
    ),
    path(
        "routing-policy-rule/<int:pk>/",
        include(get_model_urls("netbox_bgp", "routingpolicyrule")),
    ),

    # Peer Groups
    path(
        "peer-group/",
        include(get_model_urls("netbox_bgp", "bgppeergroup", detail=False)),
    ),
    path(
        "peer-group/<int:pk>/",
        include(get_model_urls("netbox_bgp", "bgppeergroup")),
    ),

    # Prefix Lists
    path(
        "prefix-list/",
        include(get_model_urls("netbox_bgp", "prefixlist", detail=False)),
    ),
    path(
        "prefix-list/<int:pk>/",
        include(get_model_urls("netbox_bgp", "prefixlist")),
    ),

    # Prefix List Rules
    path(
        "prefix-list-rule/",
        include(get_model_urls("netbox_bgp", "prefixlistrule", detail=False)),
    ),
    path(
        "prefix-list-rule/<int:pk>/",
        include(get_model_urls("netbox_bgp", "prefixlistrule")),
    ),
)



# Generated by Django 4.1.9 on 2023-09-19 05:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0028_netbox_bgp'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bgpsession',
            name='name',
            field=models.CharField(blank=True, max_length=256, null=True),
        ),
    ]



from django.db.models import Q
from django.contrib import messages
from django.shortcuts import get_object_or_404, redirect, render, reverse
from django.utils.text import slugify
from utilities.views import register_model_view, ViewTab
from netbox.views import generic
from ipam.models import ASN
from virtualization.models import VirtualMachine

from .models import (
    Community, BGPSession, RoutingPolicy,
    BGPPeerGroup, RoutingPolicyRule, PrefixList,
    PrefixListRule, CommunityList, CommunityListRule,
    ASPathList, ASPathListRule
)

from . import filtersets, forms, tables


# Community

@register_model_view(Community, "list", path="", detail=False)
class CommunityListView(generic.ObjectListView):
    queryset = Community.objects.all()
    filterset = filtersets.CommunityFilterSet
    filterset_form = forms.CommunityFilterForm
    table = tables.CommunityTable

@register_model_view(Community)
class CommunityView(generic.ObjectView):
    queryset = Community.objects.all()
    template_name = 'netbox_bgp/community.html'

@register_model_view(Community, "add", detail=False)
@register_model_view(Community, "edit")
class CommunityEditView(generic.ObjectEditView):
    queryset = Community.objects.all()
    form = forms.CommunityForm

@register_model_view(Community, "bulk_delete", path="delete", detail=False)
class CommunityBulkDeleteView(generic.BulkDeleteView):
    queryset = Community.objects.all()
    table = tables.CommunityTable

@register_model_view(Community, "bulk_edit", path="edit", detail=False)
class CommunityBulkEditView(generic.BulkEditView):
    queryset = Community.objects.all()
    filterset = filtersets.CommunityFilterSet
    table = tables.CommunityTable
    form = forms.CommunityBulkEditForm

@register_model_view(Community, "delete")
class CommunityDeleteView(generic.ObjectDeleteView):
    queryset = Community.objects.all()
    default_return_url = 'plugins:netbox_bgp:community_list'

@register_model_view(Community, "bulk_import", path="import", detail=False)
class CommunityBulkImportView(generic.BulkImportView):
    queryset = Community.objects.all()
    model_form = forms.CommunityImportForm


# Community List

@register_model_view(CommunityList, "list", path="", detail=False)
class CommunityListListView(generic.ObjectListView):
    queryset = CommunityList.objects.all()
    filterset = filtersets.CommunityListFilterSet
    filterset_form = forms.CommunityListFilterForm
    table = tables.CommunityListTable

@register_model_view(CommunityList, "add", detail=False)
@register_model_view(CommunityList, "edit")
class CommunityListEditView(generic.ObjectEditView):
    queryset = CommunityList.objects.all()
    form = forms.CommunityListForm

@register_model_view(CommunityList, "bulk_delete", path="delete", detail=False)
class CommunityListBulkDeleteView(generic.BulkDeleteView):
    queryset = CommunityList.objects.all()
    table = tables.CommunityListTable

@register_model_view(CommunityList, "bulk_edit", path="edit", detail=False)
class CommunityListBulkEditView(generic.BulkEditView):
    queryset = CommunityList.objects.all()
    filterset = filtersets.CommunityListFilterSet
    table = tables.CommunityListTable
    form = forms.CommunityListBulkEditForm

@register_model_view(CommunityList)
class CommListView(generic.ObjectView):
    queryset = CommunityList.objects.all()
    template_name = 'netbox_bgp/communitylist.html'

    def get_extra_context(self, request, instance):
        rprules = instance.cmrules.all()
        rprules_table = tables.RoutingPolicyRuleTable(rprules)
        rules = instance.commlistrules.all()
        rules_table = tables.CommunityListRuleTable(rules)
        return {
            'rules_table': rules_table,
            'rprules_table': rprules_table
        }

@register_model_view(CommunityList, "delete")
class CommunityListDeleteView(generic.ObjectDeleteView):
    queryset = CommunityList.objects.all()
    default_return_url = 'plugins:netbox_bgp:communitylist_list'

@register_model_view(CommunityList, "bulk_import", path="import", detail=False)
class CommunityListBulkImportView(generic.BulkImportView):
    queryset = CommunityList.objects.all()
    model_form = forms.CommunityListImportForm


# Community List Rule

@register_model_view(CommunityListRule, "list", path="", detail=False)
class CommunityListRuleListView(generic.ObjectListView):
    queryset = CommunityListRule.objects.all()
    filterset = filtersets.CommunityListRuleFilterSet
    # filterset_form = RoutingPolicyRuleFilterForm
    table = tables.CommunityListRuleTable
    actions = {'add': {'add'}, 'bulk_delete': {'delete'}}

@register_model_view(CommunityListRule, "add", detail=False)
@register_model_view(CommunityListRule, "edit")
class CommunityListRuleEditView(generic.ObjectEditView):
    queryset = CommunityListRule.objects.all()
    form = forms.CommunityListRuleForm

@register_model_view(CommunityListRule, "bulk_delete", path="delete", detail=False)
class CommunityListRuleBulkDeleteView(generic.BulkDeleteView):
    queryset = CommunityListRule.objects.all()
    table = tables.CommunityListRuleTable

@register_model_view(CommunityListRule, "delete", path="delete", detail=False)
class CommunityListRuleDeleteView(generic.ObjectDeleteView):
    queryset = CommunityListRule.objects.all()
    default_return_url = 'plugins:netbox_bgp:communitylistrule_list'

@register_model_view(CommunityListRule)
class CommunityListRuleView(generic.ObjectView):
    queryset = CommunityListRule.objects.all()
    template_name = 'netbox_bgp/communitylistrule.html'


# Session

@register_model_view(BGPSession, "list", path="", detail=False)
class BGPSessionListView(generic.ObjectListView):
    queryset = BGPSession.objects.all()
    filterset = filtersets.BGPSessionFilterSet
    filterset_form = forms.BGPSessionFilterForm
    table = tables.BGPSessionTable

@register_model_view(BGPSession, "edit")
class BGPSessionEditView(generic.ObjectEditView):
    queryset = BGPSession.objects.all()
    form = forms.BGPSessionForm

@register_model_view(BGPSession, "add", detail=False)
class BGPSessionAddView(generic.ObjectEditView):
    queryset = BGPSession.objects.all()
    form = forms.BGPSessionAddForm

@register_model_view(BGPSession, "bulk_import", path="import", detail=False)
class BGPSessionBulkImportView(generic.BulkImportView):
    queryset = BGPSession.objects.all()
    model_form = forms.BGPSessionImportForm

@register_model_view(BGPSession, "bulk_edit", path="edit", detail=False)
class BGPSessionBulkEditView(generic.BulkEditView):
    queryset = BGPSession.objects.all()
    filterset = filtersets.BGPSessionFilterSet
    table = tables.BGPSessionTable
    form = forms.BGPSessionBulkEditForm

@register_model_view(BGPSession, "bulk_delete", path="delete", detail=False)
class BGPSessionBulkDeleteView(generic.BulkDeleteView):
    queryset = BGPSession.objects.all()
    table = tables.BGPSessionTable

@register_model_view(BGPSession)
class BGPSessionView(generic.ObjectView):
    queryset = BGPSession.objects.all()
    template_name = 'netbox_bgp/bgpsession.html'

    def get_extra_context(self, request, instance):
        import_policies_qs = instance.import_policies.all()
        if not import_policies_qs and instance.peer_group:
            import_policies_qs = instance.peer_group.import_policies.all()
        export_policies_qs = instance.export_policies.all()
        if not export_policies_qs and instance.peer_group:
            export_policies_qs = instance.peer_group.export_policies.all()

        import_policies_table = tables.RoutingPolicyTable(
            import_policies_qs,
            orderable=False
        )
        export_policies_table = tables.RoutingPolicyTable(
            export_policies_qs,
            orderable=False
        )

        return {
            'import_policies_table': import_policies_table,
            'export_policies_table': export_policies_table
        }

@register_model_view(BGPSession, "delete")
class BGPSessionDeleteView(generic.ObjectDeleteView):
    queryset = BGPSession.objects.all()
    default_return_url = 'plugins:netbox_bgp:bgpsession_list'


# Routing Policy

@register_model_view(RoutingPolicy, "list", path="", detail=False)
class RoutingPolicyListView(generic.ObjectListView):
    queryset = RoutingPolicy.objects.all()
    filterset = filtersets.RoutingPolicyFilterSet
    filterset_form = forms.RoutingPolicyFilterForm
    table = tables.RoutingPolicyTable

@register_model_view(RoutingPolicy, "add", detail=False)
@register_model_view(RoutingPolicy, "edit")
class RoutingPolicyEditView(generic.ObjectEditView):
    queryset = RoutingPolicy.objects.all()
    form = forms.RoutingPolicyForm

@register_model_view(RoutingPolicy, "bulk_delete", path="delete", detail=False)
class RoutingPolicyBulkDeleteView(generic.BulkDeleteView):
    queryset = RoutingPolicy.objects.all()
    table = tables.RoutingPolicyTable

@register_model_view(RoutingPolicy, "bulk_edit", path="edit", detail=False)
class RoutingPolicyBulkEditView(generic.BulkEditView):
    queryset = RoutingPolicy.objects.all()
    filterset = filtersets.RoutingPolicyFilterSet
    table = tables.RoutingPolicyTable
    form = forms.RoutingPolicyBulkEditForm

@register_model_view(RoutingPolicy)
class RoutingPolicyView(generic.ObjectView):
    queryset = RoutingPolicy.objects.all()
    template_name = 'netbox_bgp/routingpolicy.html'

    def get_extra_context(self, request, instance):
        sess = BGPSession.objects.filter(
            Q(import_policies=instance)
            | Q(export_policies=instance)
            | Q(peer_group__in=instance.group_import_policies.all())
            | Q(peer_group__in=instance.group_export_policies.all())
        )
        sess = sess.distinct()
        sess_table = tables.BGPSessionTable(sess)
        rules = instance.rules.all()
        rules_table = tables.RoutingPolicyRuleTable(rules)
        return {
            'rules_table': rules_table,
            'related_session_table': sess_table
        }

@register_model_view(RoutingPolicy, "delete")
class RoutingPolicyDeleteView(generic.ObjectDeleteView):
    queryset = RoutingPolicy.objects.all()
    default_return_url = 'plugins:netbox_bgp:routingpolicy_list'

@register_model_view(RoutingPolicy, "bulk_import", path="import", detail=False)
class RoutingPolicyBulkImportView(generic.BulkImportView):
    queryset = RoutingPolicy.objects.all()
    model_form = forms.RoutingPolicyImportForm


# Routing Policy Rule

@register_model_view(RoutingPolicyRule, "list", path="", detail=False)
class RoutingPolicyRuleListView(generic.ObjectListView):
    queryset = RoutingPolicyRule.objects.all()
    filterset = filtersets.RoutingPolicyRuleFilterSet
    # filterset_form = RoutingPolicyRuleFilterForm
    table = tables.RoutingPolicyRuleTable
    actions = {'add': {'add'}, 'bulk_delete': {'delete'}}

@register_model_view(RoutingPolicyRule, "add", detail=False)
@register_model_view(RoutingPolicyRule, "edit")
class RoutingPolicyRuleEditView(generic.ObjectEditView):
    queryset = RoutingPolicyRule.objects.all()
    form = forms.RoutingPolicyRuleForm

@register_model_view(RoutingPolicyRule, "delete")
class RoutingPolicyRuleDeleteView(generic.ObjectDeleteView):
    queryset = RoutingPolicyRule.objects.all()
    default_return_url = 'plugins:netbox_bgp:routingpolicyrule_list'

@register_model_view(RoutingPolicyRule, "bulk_delete", path="delete", detail=False)
class RoutingPolicyRuleBulkDeleteView(generic.BulkDeleteView):
    queryset = RoutingPolicyRule.objects.all()
    table = tables.RoutingPolicyRuleTable

@register_model_view(RoutingPolicyRule)
class RoutingPolicyRuleView(generic.ObjectView):
    queryset = RoutingPolicyRule.objects.all()
    template_name = 'netbox_bgp/routingpolicyrule.html'

    def get_extra_context(self, request, instance):
        if request.GET.get('format') in ['json', 'yaml']:
            format = request.GET.get('format')
            if request.user.is_authenticated:
                request.user.config.set('data_format', format, commit=True)
        elif request.user.is_authenticated:
            format = request.user.config.get('data_format', 'json')
        else:
            format = 'json'

        return {
            'format': format,
        }


@register_model_view(RoutingPolicyRule, "bulk_import", path="import", detail=False)
class RoutingPolicyRuleImportView(generic.BulkImportView):
    queryset = RoutingPolicyRule.objects.all()
    model_form = forms.RoutingPolicyRuleImportForm

# Peer Group

@register_model_view(BGPPeerGroup, "list", path="", detail=False)
class BGPPeerGroupListView(generic.ObjectListView):
    queryset = BGPPeerGroup.objects.all()
    filterset = filtersets.BGPPeerGroupFilterSet
    filterset_form = forms.BGPPeerGroupFilterForm
    table = tables.BGPPeerGroupTable

@register_model_view(BGPPeerGroup, "add", detail=False)
@register_model_view(BGPPeerGroup, "edit")
class BGPPeerGroupEditView(generic.ObjectEditView):
    queryset = BGPPeerGroup.objects.all()
    form = forms.BGPPeerGroupForm

@register_model_view(BGPPeerGroup, "bulk_delete", path="delete", detail=False)
class BGPPeerGroupBulkDeleteView(generic.BulkDeleteView):
    queryset = BGPPeerGroup.objects.all()
    table = tables.BGPPeerGroupTable

@register_model_view(BGPPeerGroup)
class BGPPeerGroupView(generic.ObjectView):
    queryset = BGPPeerGroup.objects.all()
    template_name = 'netbox_bgp/bgppeergroup.html'

    def get_extra_context(self, request, instance):
        import_policies_table = tables.RoutingPolicyTable(
            instance.import_policies.all(),
            orderable=False
        )
        export_policies_table = tables.RoutingPolicyTable(
            instance.export_policies.all(),
            orderable=False
        )

        sess = BGPSession.objects.filter(peer_group=instance)
        sess = sess.distinct()
        sess_table = tables.BGPSessionTable(sess)
        return {
            'import_policies_table': import_policies_table,
            'export_policies_table': export_policies_table,
            'related_session_table': sess_table
        }

@register_model_view(BGPPeerGroup, "delete")
class BGPPeerGroupDeleteView(generic.ObjectDeleteView):
    queryset = BGPPeerGroup.objects.all()
    default_return_url = 'plugins:netbox_bgp:bgppeergroup_list'

@register_model_view(BGPPeerGroup, "bulk_import", path="import", detail=False)
class BGPPeerGroupBulkImportView(generic.BulkImportView):
    queryset = BGPPeerGroup.objects.all()
    model_form = forms.BGPPeerGroupImportForm

@register_model_view(BGPPeerGroup, "bulk_edit", path="edit", detail=False)
class BGPPeerGroupBulkEditView(generic.BulkEditView):
    queryset = BGPPeerGroup.objects.all()
    filterset = filtersets.BGPPeerGroupFilterSet
    table = tables.BGPPeerGroupTable
    form = forms.BGPPeerGroupBulkEditForm


# Prefix List

@register_model_view(PrefixList, "list", path="", detail=False)
class PrefixListListView(generic.ObjectListView):
    queryset = PrefixList.objects.all()
    filterset = filtersets.PrefixListFilterSet
    filterset_form = forms.PrefixListFilterForm
    table = tables.PrefixListTable

@register_model_view(PrefixList, "add", detail=False)
@register_model_view(PrefixList, "edit")
class PrefixListEditView(generic.ObjectEditView):
    queryset = PrefixList.objects.all()
    form = forms.PrefixListForm

@register_model_view(PrefixList, "bulk_delete", path="delete", detail=False)
class PrefixListBulkDeleteView(generic.BulkDeleteView):
    queryset = PrefixList.objects.all()
    table = tables.PrefixListTable

@register_model_view(PrefixList, "bulk_edit", path="edit", detail=False)
class PrefixListBulkEditView(generic.BulkEditView):
    queryset = PrefixList.objects.all()
    filterset = filtersets.PrefixListFilterSet
    table = tables.PrefixListTable
    form = forms.PrefixListBulkEditForm

@register_model_view(PrefixList)
class PrefixListView(generic.ObjectView):
    queryset = PrefixList.objects.all()
    template_name = 'netbox_bgp/prefixlist.html'

    def get_extra_context(self, request, instance):
        rprules = instance.plrules.all()
        rprules_table = tables.RoutingPolicyRuleTable(rprules)
        rules = instance.prefrules.all()
        rules_table = tables.PrefixListRuleTable(rules)

        sess = instance.session_prefix_in.all() | instance.session_prefix_out.all()
        sess_table = tables.BGPSessionTable(sess)
        return {
            'rules_table': rules_table,
            'rprules_table': rprules_table,
            'sess_table': sess_table,
        }

@register_model_view(PrefixList, "delete")
class PrefixListDeleteView(generic.ObjectDeleteView):
    queryset = PrefixList.objects.all()
    default_return_url = 'plugins:netbox_bgp:prefixlist_list'

@register_model_view(PrefixList, "bulk_import", path="import", detail=False)
class PrefixListBulkImportView(generic.BulkImportView):
    queryset = PrefixList.objects.all()
    model_form = forms.PrefixListImportForm


# Prefix List Rule

@register_model_view(PrefixListRule, "list", path="", detail=False)
class PrefixListRuleListView(generic.ObjectListView):
    queryset = PrefixListRule.objects.all()
    filterset = filtersets.PrefixListRuleFilterSet
    # filterset_form = RoutingPolicyRuleFilterForm
    table = tables.PrefixListRuleTable
    actions = {'add': {'add'}, 'bulk_delete': {'delete'}}

@register_model_view(PrefixListRule, "add", detail=False)
@register_model_view(PrefixListRule, "edit")
class PrefixListRuleEditView(generic.ObjectEditView):
    queryset = PrefixListRule.objects.all()
    form = forms.PrefixListRuleForm

@register_model_view(PrefixListRule, "bulk_delete", path="delete", detail=False)
class PrefixListRuleBulkDeleteView(generic.BulkDeleteView):
    queryset = PrefixListRule.objects.all()
    table = tables.PrefixListRuleTable

@register_model_view(PrefixListRule, "delete")
class PrefixListRuleDeleteView(generic.ObjectDeleteView):
    queryset = PrefixListRule.objects.all()
    default_return_url = 'plugins:netbox_bgp:prefixlistrule_list'

@register_model_view(PrefixListRule)
class PrefixListRuleView(generic.ObjectView):
    queryset = PrefixListRule.objects.all()
    template_name = 'netbox_bgp/prefixlistrule.html'


@register_model_view(PrefixListRule, "bulk_import", path="import", detail=False)
class PrefixListRuleViewImportView(generic.BulkImportView):
    queryset = PrefixListRule.objects.all()
    model_form = forms.PrefixListRuleImportForm


# Viewtab for Virtual Machine

@register_model_view(VirtualMachine, name='bgpsessions', path='bgpsessions')
class VMBGPSessionView(generic.ObjectChildrenView):
    queryset = VirtualMachine.objects.all().prefetch_related('bgpsession_set')
    child_model = BGPSession
    table = tables.BGPSessionTable
    template_name = "generic/object_children.html"
    tab = ViewTab(
        label='BGP Sessions',
        badge=lambda obj:  obj.bgpsession_set.count(),
        hide_if_empty = True
    )

    def get_children(self, request, parent):
        return parent.bgpsession_set.all()

# AS Path List

@register_model_view(ASPathList, "list", path="", detail=False)
class ASPathListListView(generic.ObjectListView):
    queryset = ASPathList.objects.all()
    filterset = filtersets.ASPathListFilterSet
    filterset_form = forms.ASPathListFilterForm
    table = tables.ASPathListTable

@register_model_view(ASPathList, "add", detail=False)
@register_model_view(ASPathList, "edit")
class ASPathListEditView(generic.ObjectEditView):
    queryset = ASPathList.objects.all()
    form = forms.ASPathListForm

@register_model_view(ASPathList, "bulk_delete", path="delete", detail=False)
class ASPathListBulkDeleteView(generic.BulkDeleteView):
    queryset = ASPathList.objects.all()
    table = tables.ASPathListTable

@register_model_view(ASPathList, "bulk_edit", path="edit", detail=False)
class ASPathListBulkEditView(generic.BulkEditView):
    queryset = ASPathList.objects.all()
    filterset = filtersets.ASPathListFilterSet
    table = tables.ASPathListTable
    form = forms.ASPathListBulkEditForm

@register_model_view(ASPathList)
class ASPathListView(generic.ObjectView):
    queryset = ASPathList.objects.all()
    template_name = 'netbox_bgp/aspathlist.html'

    def get_extra_context(self, request, instance):
        rprules = instance.aspathrules.all()
        rprules_table = tables.RoutingPolicyRuleTable(rprules)
        rules = instance.aspathlistrules.all()
        rules_table = tables.ASPathListRuleTable(rules)
        return {
            'rules_table': rules_table,
            'rprules_table': rprules_table
        }

@register_model_view(ASPathList, "delete")
class ASPathListDeleteView(generic.ObjectDeleteView):
    queryset = ASPathList.objects.all()
    default_return_url = 'plugins:netbox_bgp:aspathlist_list'

@register_model_view(ASPathList, "bulk_import", path="import", detail=False)
class ASPathListBulkImportView(generic.BulkImportView):
    queryset = ASPathList.objects.all()
    model_form = forms.ASPathListImportForm

# AS Path List Rule 

@register_model_view(ASPathListRule, "list", path="", detail=False)
class ASPathListRuleListView(generic.ObjectListView):
    queryset = ASPathListRule.objects.all()
    filterset = filtersets.ASPathListRuleFilterSet
    # filterset_form = ASPathListRuleFilterForm
    table = tables.ASPathListRuleTable
    actions = {'add': {'add'}, 'bulk_delete': {'delete'}}


@register_model_view(ASPathListRule, "add", detail=False)
@register_model_view(ASPathListRule, "edit")
class ASPathListRuleEditView(generic.ObjectEditView):
    queryset = ASPathListRule.objects.all()
    form = forms.ASPathListRuleForm


@register_model_view(ASPathListRule, "bulk_delete", path="delete", detail=False)
class ASPathListRuleBulkDeleteView(generic.BulkDeleteView):
    queryset = ASPathListRule.objects.all()
    table = tables.ASPathListRuleTable


@register_model_view(ASPathListRule, "delete")
class ASPathListRuleDeleteView(generic.ObjectDeleteView):
    queryset = ASPathListRule.objects.all()
    default_return_url = 'plugins:netbox_bgp:aspathlistrule_list'


@register_model_view(ASPathListRule, "bulk_import", path="import", detail=False)
class ASPathListRuleBulkImportView(generic.BulkImportView):
    queryset = ASPathListRule.objects.all()
    model_form = forms.ASPathListRuleImportForm


@register_model_view(ASPathListRule)
class ASPathListRuleView(generic.ObjectView):
    queryset = ASPathListRule.objects.all()
    template_name = 'netbox_bgp/aspathlistrule.html'



import json

from django.urls import reverse
from utilities.testing import APITestCase, APIViewTestCases

from tenancy.models import Tenant
from dcim.models import Site, DeviceRole, DeviceType, Manufacturer, Device, Interface
from ipam.models import IPAddress, ASN, RIR, Prefix

from netbox_bgp.models import (
    Community,
    CommunityList,
    CommunityListRule,
    BGPPeerGroup,
    BGPSession,
    RoutingPolicy,
    RoutingPolicyRule,
    PrefixList,
    PrefixListRule,
)

from netbox_bgp.choices import (
    SessionStatusChoices,
    IPAddressFamilyChoices,
    ActionChoices,
)


class CommunityAPITestCase(
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
    APIViewTestCases.GraphQLTestCase,
):
    model = Community
    view_namespace = "plugins-api:netbox_bgp"
    brief_fields = ["description", "display", "id", "url", "value"]
    graphql_base_name = "netbox_bgp_community"

    create_data = [
        {"value": "65001:65000"},
        {"value": "65002:65001"},
        {"value": "65003:65002"},
    ]

    bulk_update_data = {
        "description": "Test Community desc",
    }

    @classmethod
    def setUpTestData(cls):
        communities = (
            Community(value="65000:65000"),
            Community(value="65000:65001"),
            Community(value="65000:65002"),
        )
        Community.objects.bulk_create(communities)


class CommunityListAPITestCase(
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
    APIViewTestCases.GraphQLTestCase,
):
    model = CommunityList
    view_namespace = "plugins-api:netbox_bgp"
    brief_fields = ["description", "display", "id", "name", "url"]
    graphql_base_name = "netbox_bgp_communitylist"

    create_data = [
        {"name": "CL1", "description": "cl1_api"},
        {"name": "CL2", "description": "cl2_api"},
        {"name": "CL3", "description": "cl3_api"},
    ]

    bulk_update_data = {
        "description": "Test Community List desc",
    }
    user_permissions = [
        "netbox_bgp.view_community",
    ]

    @classmethod
    def setUpTestData(cls):
        communitylists = (
            CommunityList(name="CL4", description="cl4"),
            CommunityList(name="CL5", description="cl5"),
            CommunityList(name="CL6", description="cl6"),
        )
        CommunityList.objects.bulk_create(communitylists)


class CommunityListRuleAPITestCase(
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
    APIViewTestCases.GraphQLTestCase,
):
    model = CommunityListRule
    view_namespace = "plugins-api:netbox_bgp"
    brief_fields = ["description", "display", "id"]
    graphql_base_name = "netbox_bgp_communitylist_rule"

    bulk_update_data = {
        "description": "Test Community List rules desc",
        "action": "deny",
    }

    user_permissions = [
        "netbox_bgp.view_communitylist",
        "netbox_bgp.view_community",
    ]


    @classmethod
    def setUpTestData(cls):
        com_list1 = CommunityList.objects.create(
            name="community list 1", description="community list 1"
        )
        com_list2 = CommunityList.objects.create(
            name="community list 2", description="community list 2"
        )
        com1 = Community.objects.create(value="65001:65004", description="community1")
        com2 = Community.objects.create(value="65002:65005", description="community2")
        com3 = Community.objects.create(value="65003:65006", description="community3")

        communitylistrules = (
            CommunityListRule(
                community_list=com_list1,
                community=com1,
                action=ActionChoices._choices[0][0],
                description="rule1",
            ),
            CommunityListRule(
                community_list=com_list1,
                community=com2,
                action=ActionChoices._choices[0][0],
                description="rule2",
            ),
            CommunityListRule(
                community_list=com_list1,
                community=com3,
                action=ActionChoices._choices[0][1],
                description="rule3",
            ),
        )
        CommunityListRule.objects.bulk_create(communitylistrules)

        cls.create_data = [
            {
                "community_list": com_list2.id,
                "description": "rule4",
                "community": com1.id,
                "action": "permit",
                "comments": "rule4",
            },
            {
                "community_list": com_list2.id,
                "description": "rule5",
                "community": com2.id,
                "action": "permit",
                "comments": "rule5",
            },
            {
                "community_list": com_list2.id,
                "description": "rule6",
                "community": com3.id,
                "action": "deny",
                "comments": "rule6",
            },
        ]


class BGPPeerGroupAPITestCase(
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
    APIViewTestCases.GraphQLTestCase,
):
    model = BGPPeerGroup
    view_namespace = "plugins-api:netbox_bgp"
    brief_fields = ["description", "display", "id", "name", "url"]
    graphql_base_name = "netbox_bgp_peer_group"

    create_data = [
        {
            "name": "test_peer_group",
            "description": "peer_group_desc",
            "comments": "peer_group_comment1",
        },
        {
            "name": "test_peer_group2",
            "description": "peer_group_desc2",
            "comments": "peer_group_comment2",
        },
        {
            "name": "test_peer_group3",
            "description": "peer_group_desc3",
            "comments": "peer_group_comment3",
        },
    ]

    bulk_update_data = {
        "description": "Test Community desc",
    }

    @classmethod
    def setUpTestData(cls):
        peer_groups = (
            BGPPeerGroup(
                name="peer group 1", description="peer group 1", comments="peer group 1"
            ),
            BGPPeerGroup(
                name="peer group 2", description="peer group 2", comments="peer group 2"
            ),
            BGPPeerGroup(
                name="peer group 3", description="peer group 3", comments="peer group 3"
            ),
        )
        BGPPeerGroup.objects.bulk_create(peer_groups)


class BGPSessionAPITestCase(
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
    APIViewTestCases.GraphQLTestCase,
):
    model = BGPSession
    view_namespace = "plugins-api:netbox_bgp"
    brief_fields = ["description", "display", "id", "name", "url"]
    graphql_base_name = "netbox_bgp_session"

    bulk_update_data = {
        "description": "Test BGP session desc",
    }
    user_permissions = [
        "ipam.view_ipaddress",
        "ipam.view_asn"
    ]


    @classmethod
    def setUpTestData(cls):
        site = Site.objects.create(name="test", slug="test")
        manufacturer = Manufacturer.objects.create(name="Juniper", slug="juniper")
        device_role = DeviceRole.objects.create(name="Firewall", slug="firewall")
        device_type = DeviceType.objects.create(
            slug="srx3600", model="SRX3600", manufacturer=manufacturer
        )
        device = Device.objects.create(
            device_type=device_type, name="device1", role=device_role, site=site
        )
        device2 = Device.objects.create(
            device_type=device_type, name="device2", role=device_role, site=site
        )
        intf = Interface.objects.create(name="test_intf1", device=device)
        intf2 = Interface.objects.create(name="test_intf2", device=device)
        intf3 = Interface.objects.create(name="test_intf3", device=device)
        local_ip = IPAddress.objects.create(address="1.1.1.1/32")
        remote_ip1 = IPAddress.objects.create(address="2.2.2.2/32")
        remote_ip2 = IPAddress.objects.create(address="3.3.3.3/32")
        remote_ip3 = IPAddress.objects.create(address="4.4.4.4/32")
        intf.ip_addresses.add(local_ip)
        rir = RIR.objects.create(name="rir")
        local_as = ASN.objects.create(asn=65002, rir=rir, description="local_as")
        remote_as = ASN.objects.create(asn=65003, rir=rir, description="remote_as")
        peer_group = BGPPeerGroup.objects.create(
            name="peer_group", description="peer_group_description"
        )
        rp = RoutingPolicy.objects.create(
            name="rp1", description="test_rp", comments="comments_routing_policy"
        )
        pl1 = PrefixList.objects.create(
            name="pl1",
            description="test_pl",
            family=IPAddressFamilyChoices.FAMILY_4,
            comments="comments_pl",
        )

        sessions = (
            BGPSession(
                name="Session1",
                description="Session1",
                comments="Session1",
                local_as=local_as,
                remote_as=remote_as,
                local_address=local_ip,
                remote_address=remote_ip1,
                peer_group=peer_group,
                status=SessionStatusChoices.STATUS_ACTIVE,
            ),
            BGPSession(
                name="Session2",
                description="Session2",
                comments="Session2",
                local_as=local_as,
                remote_as=remote_as,
                local_address=local_ip,
                remote_address=remote_ip1,
                peer_group=peer_group,
                status=SessionStatusChoices.STATUS_ACTIVE,
            ),
            BGPSession(
                name="Session3",
                description="Session3",
                comments="Session3",
                local_as=local_as,
                remote_as=remote_as,
                local_address=local_ip,
                remote_address=remote_ip1,
                peer_group=peer_group,
                status=SessionStatusChoices.STATUS_ACTIVE,
                prefix_list_in=pl1,
            ),
        )
        BGPSession.objects.bulk_create(sessions)

        # sessions[0].import_policies.add(rp)

        cls.create_data = [
            {
                "name": "test_session1",
                "description": "test_session1",
                "comments": "test_session1",
                "local_as": local_as.id,
                "remote_as": remote_as.id,
                "device": device2.id,
                "local_address": local_ip.id,
                "remote_address": remote_ip1.id,
                "export_policies": [rp.id],
            },
            {
                "name": "test_session2",
                "description": "test_session2",
                "comments": "test_session2",
                "local_as": local_as.id,
                "remote_as": remote_as.id,
                "device": device2.id,
                "local_address": local_ip.id,
                "remote_address": remote_ip2.id,
                "import_policies": [rp.id],
            },
            {
                "name": "test_session3",
                "description": "test_session3",
                "comments": "test_session3",
                "local_as": local_as.id,
                "remote_as": remote_as.id,
                "device": device2.id,
                "local_address": local_ip.id,
                "remote_address": remote_ip3.id,
                "prefix_list_in": pl1.id,
            },
        ]


class RoutingPolicyAPITestCase(
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
    APIViewTestCases.GraphQLTestCase,
):
    model = RoutingPolicy
    view_namespace = "plugins-api:netbox_bgp"
    brief_fields = ["description", "display", "id", "name", "url"]
    graphql_base_name = "netbox_bgp_routing_policy"

    create_data = [
        {
            "name": "test_routing_policy",
            "description": "routing_policy_desc",
            "comments": "routing_policy_comment1",
        },
        {
            "name": "test_routing_policy2",
            "description": "routing_policy_desc2",
            "comments": "routing_policy_comment2",
        },
        {
            "name": "test_routing_policy3",
            "description": "routing_policy_desc3",
            "comments": "routing_policy_comment3",
        },
    ]

    bulk_update_data = {
        "description": "Test Routing policy desc",
    }

    @classmethod
    def setUpTestData(cls):
        routing_policies = (
            RoutingPolicy(
                name="Route-map 1", description="Route-map 1", comments="Route-map 1"
            ),
            RoutingPolicy(
                name="Route-map 2", description="Route-map 2", comments="Route-map 2"
            ),
            RoutingPolicy(
                name="Route-map 3", description="Route-map 3", comments="Route-map 3"
            ),
        )
        RoutingPolicy.objects.bulk_create(routing_policies)


class RoutingPolicyRuleAPITestCase(
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
    APIViewTestCases.GraphQLTestCase,
):
    model = RoutingPolicyRule
    view_namespace = "plugins-api:netbox_bgp"
    brief_fields = ["description", "display", "id"]
    graphql_base_name = "netbox_bgp_routing_policy_rule"

    bulk_update_data = {
        "description": "Test Routing policy rules desc",
        "action": "deny",
    }

    user_permissions = [
        "netbox_bgp.view_routingpolicy",
    ]


    @classmethod
    def setUpTestData(cls):
        rp1 = RoutingPolicy.objects.create(
            name="rp1", description="test_rp1", comments="comments_routing_policy1"
        )
        rp2 = RoutingPolicy.objects.create(
            name="rp2", description="test_rp2", comments="comments_routing_policy2"
        )
        pl1 = PrefixList.objects.create(
            name="pl1", description="test_pl", family=IPAddressFamilyChoices.FAMILY_4
        )
        pl2 = PrefixList.objects.create(
            name="pl1", description="test_pl", family=IPAddressFamilyChoices.FAMILY_6
        )
        com1 = Community.objects.create(value="65000:65000")
        com_list1 = CommunityList.objects.create(
            name="community list 1", description="community list 1"
        )

        routing_policies_rules = (
            RoutingPolicyRule(
                routing_policy=rp1,
                index=10,
                action=ActionChoices._choices[0][0],
                description="Rule1",
            ),
            RoutingPolicyRule(
                routing_policy=rp1,
                index=20,
                action=ActionChoices._choices[0][0],
                description="Rule2",
            ),
            RoutingPolicyRule(
                routing_policy=rp1,
                index=30,
                action=ActionChoices._choices[0][1],
                description="Rule3",
            ),
        )
        RoutingPolicyRule.objects.bulk_create(routing_policies_rules)

        cls.create_data = [
            {
                "routing_policy": rp2.id,
                "description": "rule4",
                "index": 10,
                "action": "permit",
                "comments": "rule4",
                "match_ip_address": [pl1.id],
            },
            {
                "routing_policy": rp2.id,
                "description": "rule5",
                "index": 20,
                "action": "permit",
                "comments": "rule5",
                "match_ipv6_address": [pl2.id],
                "set_actions": "{'set': 'origin incomplete'",
            },
            {
                "routing_policy": rp2.id,
                "description": "rule6",
                "index": 30,
                "action": "deny",
                "comments": "rule6",
                "match_community": [com1.id],
            },
            {
                "routing_policy": rp2.id,
                "description": "rule7",
                "index": 40,
                "action": "deny",
                "comments": "rule6",
                "match_community_list": [com_list1.id],
            },
        ]


class PrefixListAPITestCase(
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
    APIViewTestCases.GraphQLTestCase,
):
    model = PrefixList
    view_namespace = "plugins-api:netbox_bgp"
    brief_fields = ["description", "display", "id", "name", "url"]
    graphql_base_name = "netbox_bgp_prefixlist"

    create_data = [
        {
            "name": "test_prefix_list",
            "description": "prefix_list_desc",
            "comments": "prefix_list_comment1",
            "family": "ipv4",
        },
        {
            "name": "test_prefix_list2",
            "description": "prefix_list_desc2",
            "comments": "prefix_list_comment2",
            "family": "ipv4",
        },
        {
            "name": "test_prefix_list3",
            "description": "prefix_list_desc3",
            "comments": "prefix_list_comment3",
            "family": "ipv6",
        },
    ]

    bulk_update_data = {
        "description": "Test Prefix list desc",
    }

    @classmethod
    def setUpTestData(cls):

        prefix_lists = (
            PrefixList(
                name="prefix_list 1",
                description="prefix_list 1",
                comments="prefix_list 1",
                family=IPAddressFamilyChoices.FAMILY_4,
            ),
            PrefixList(
                name="prefix_list 2",
                description="prefix_list 2",
                comments="prefix_list 2",
                family=IPAddressFamilyChoices.FAMILY_6,
            ),
            PrefixList(
                name="prefix_list 3",
                description="prefix_list 3",
                comments="prefix_list 3",
                family=IPAddressFamilyChoices.FAMILY_4,
            ),
        )
        PrefixList.objects.bulk_create(prefix_lists)


class PrefixListRuleAPITestCase(
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
    APIViewTestCases.GraphQLTestCase,
):
    model = PrefixListRule
    view_namespace = "plugins-api:netbox_bgp"
    brief_fields = ["description", "display", "id"]
    graphql_base_name = "netbox_bgp_prefixlist_rule"

    user_permissions = [
        "netbox_bgp.view_prefixlist",
    ]

    bulk_update_data = {"description": "Test Prefix list rules desc", "action": "deny"}

    @classmethod
    def setUpTestData(cls):
        pl1 = PrefixList.objects.create(
            name="pl1", description="test_pl1", family=IPAddressFamilyChoices.FAMILY_4
        )
        pl2 = PrefixList.objects.create(
            name="pl2", description="test_pl2", family=IPAddressFamilyChoices.FAMILY_4
        )
        subnet1 = Prefix.objects.create(prefix="10.0.0.0/24")
        subnet2 = Prefix.objects.create(prefix="10.0.0.0/24")

        prefix_list_rules = (
            PrefixListRule(
                prefix_list=pl1,
                index=10,
                action=ActionChoices._choices[0][0],
                prefix=subnet1,
                ge=24,
                le=32,
                description="pl_rule_1",
                comments="pl_rule_1",
            ),
            PrefixListRule(
                prefix_list=pl1,
                index=20,
                action=ActionChoices._choices[0][0],
                prefix=subnet2,
                ge=24,
                le=32,
                description="pl_rule_2",
                comments="pl_rule_2",
            ),
            PrefixListRule(
                prefix_list=pl1,
                index=30,
                action=ActionChoices._choices[0][1],
                prefix_custom="0.0.0.0/0",
                ge=8,
                le=32,
                description="pl_rule_3",
                comments="pl_rule_3",
            ),
        )
        PrefixListRule.objects.bulk_create(prefix_list_rules)

        cls.create_data = [
            {
                "prefix_list": pl2.id,
                "description": "rule4",
                "index": 10,
                "action": "permit",
                "comments": "rule4",
                "prefix": subnet1.id,
                "ge": 25,
                "le": 32,
            },
            {
                "prefix_list": pl2.id,
                "description": "rule5",
                "index": 20,
                "action": "permit",
                "comments": "rule5",
                "prefix": subnet2.id,
                "ge": 26,
                "le": 32,
            },
            {
                "prefix_list": pl2.id,
                "description": "rule6",
                "index": 30,
                "action": "deny",
                "comments": "rule6",
                "prefix_custom": "0.0.0.0/0",
            },
        ]


class TestAPISchema(APITestCase):
    def test_api_schema(self):
        url = reverse("plugins-api:netbox_bgp-api:api-root")
        response = self.client.get(f"{url}?format=api", **self.header)

        self.assertEqual(response.status_code, 200)



import strawberry
import strawberry_django
from strawberry.scalars import ID
from strawberry_django import FilterLookup

from typing import Annotated
from netbox.graphql.filter_mixins import NetBoxModelFilterMixin
from tenancy.graphql.filter_mixins import TenancyFilterMixin
from ipam.graphql.filters import IPAddressFilter, ASNFilter
from dcim.graphql.filters import DeviceFilter

from netbox_bgp.models import (
    Community,
    BGPSession,
    RoutingPolicy,
    BGPPeerGroup,
    RoutingPolicyRule,
    PrefixList,
    PrefixListRule,
    CommunityList,
    CommunityListRule,
    ASPathList,
    ASPathListRule
)

from netbox_bgp.filtersets import (
    CommunityFilterSet,
    BGPSessionFilterSet,
    BGPPeerGroupFilterSet,
    RoutingPolicyFilterSet,
    RoutingPolicyRuleFilterSet,
    PrefixListFilterSet,
    PrefixListRuleFilterSet,
    CommunityListFilterSet,
    CommunityListRuleFilterSet,
    ASPathListFilterSet,
    ASPathListRuleFilterSet
)

from netbox_bgp.graphql.enums import (
    NetBoxBGPCommunityStatusEnum,
    NetBoxBGPSessionStatusEnum,
    NetBoxBGPIPAddressFamilyEnum,
    NetBoxBGPActionEnum
)


__all__ = (
    "NetBoxBGPCommunityFilter",
    "NetBoxBGPSessionFilter",
    "NetBoxBGPBGPPeerGroupFilter",
    "NetBoxBGPRoutingPolicyFilter",
    "NetBoxBGPRoutingPolicyRuleFilter",
    "NetBoxBGPPrefixListFilter",
    "NetBoxBGPPrefixListRuleFilter",
    "NetBoxBGPCommunityListFilter",
    "NetBoxBGPCommunityListRuleFilter",
    "NetBoxBGPASPathListFilter",
    "NetBoxBGPASPathListRuleFilter"
)

@strawberry_django.filter_type(ASPathList, lookups=True)
class NetBoxBGPASPathListFilter(NetBoxModelFilterMixin):
    name: FilterLookup[str] | None = strawberry_django.filter_field()
    description: FilterLookup[str] | None = strawberry_django.filter_field()

@strawberry_django.filter_type(ASPathListRule, lookups=True)
class NetBoxBGPASPathListRuleFilter(NetBoxModelFilterMixin):
    value: FilterLookup[str] | None = strawberry_django.filter_field()
    aspath_list: (
        Annotated[
            "NetBoxBGPASPathListFilter", strawberry.lazy("netbox_bgp.graphql.filters")
        ]
        | None
    ) = strawberry_django.filter_field()
    aspath_list_id: ID | None = strawberry_django.filter_field()
    action: (
        Annotated[
            "NetBoxBGPActionEnum", strawberry.lazy("netbox_bgp.graphql.enums")
        ]
        | None
    ) = strawberry_django.filter_field()

@strawberry_django.filter_type(Community, lookups=True)
class NetBoxBGPCommunityFilter(TenancyFilterMixin, NetBoxModelFilterMixin):
    value: FilterLookup[str] | None = strawberry_django.filter_field()
    description: FilterLookup[str] | None = strawberry_django.filter_field()
    status: (
        Annotated[
            "NetBoxBGPCommunityStatusEnum", strawberry.lazy("netbox_bgp.graphql.enums")
        ]
        | None
    ) = strawberry_django.filter_field()


@strawberry_django.filter_type(BGPSession, lookups=True)
class NetBoxBGPSessionFilter(TenancyFilterMixin, NetBoxModelFilterMixin):
    name: FilterLookup[str] | None = strawberry_django.filter_field()
    description: FilterLookup[str] | None = strawberry_django.filter_field()
    status: (
        Annotated[
            "NetBoxBGPSessionStatusEnum", strawberry.lazy("netbox_bgp.graphql.enums")
        ]
        | None
    ) = strawberry_django.filter_field()

    remote_as: (
        Annotated["ASNFilter", strawberry.lazy("ipam.graphql.filters")] | None
    ) = strawberry_django.filter_field()
    remote_as_id: ID | None = strawberry_django.filter_field()

    local_as: (
        Annotated["ASNFilter", strawberry.lazy("ipam.graphql.filters")] | None
    ) = strawberry_django.filter_field()
    local_as_id: ID | None = strawberry_django.filter_field()

    local_address: (
        Annotated["IPAddressFilter", strawberry.lazy("ipam.graphql.filters")] | None
    ) = strawberry_django.filter_field()
    local_address_id: ID | None = strawberry_django.filter_field()
    
    remote_address: (
        Annotated["IPAddressFilter", strawberry.lazy("ipam.graphql.filters")] | None
    ) = strawberry_django.filter_field()
    remote_address_id: ID | None = strawberry_django.filter_field()
    
    device: (
        Annotated["DeviceFilter", strawberry.lazy("dcim.graphql.filters")] | None
    ) = strawberry_django.filter_field()
    device_id: ID | None = strawberry_django.filter_field()

    peer_group: (
        Annotated[
            "NetBoxBGPBGPPeerGroupFilter", strawberry.lazy("netbox_bgp.graphql.filters")
        ]
        | None
    ) = strawberry_django.filter_field()

    import_policies: (
        Annotated[
            "NetBoxBGPRoutingPolicyFilter", strawberry.lazy("netbox_bgp.graphql.filters")
        ]
        | None
    ) = strawberry_django.filter_field()

    export_policies: (
        Annotated[
            "NetBoxBGPRoutingPolicyFilter", strawberry.lazy("netbox_bgp.graphql.filters")
        ]
        | None
    ) = strawberry_django.filter_field()




@strawberry_django.filter_type(BGPPeerGroup, lookups=True)
class NetBoxBGPBGPPeerGroupFilter(NetBoxModelFilterMixin):
    name: FilterLookup[str] | None = strawberry_django.filter_field()
    description: FilterLookup[str] | None = strawberry_django.filter_field()

@strawberry_django.filter_type(RoutingPolicy, lookups=True)
class NetBoxBGPRoutingPolicyFilter(NetBoxModelFilterMixin):
    name: FilterLookup[str] | None = strawberry_django.filter_field()
    description: FilterLookup[str] | None = strawberry_django.filter_field()

@strawberry_django.filter_type(RoutingPolicyRule, lookups=True)
class NetBoxBGPRoutingPolicyRuleFilter(NetBoxModelFilterMixin):
    description: FilterLookup[str] | None = strawberry_django.filter_field()   
    routing_policy: (
        Annotated[
            "NetBoxBGPRoutingPolicyFilter", strawberry.lazy("netbox_bgp.graphql.filters")
        ]
        | None
    ) = strawberry_django.filter_field()
    routing_policy_id: ID | None = strawberry_django.filter_field()
    action: (
        Annotated[
            "NetBoxBGPActionEnum", strawberry.lazy("netbox_bgp.graphql.enums")
        ]
        | None
    ) = strawberry_django.filter_field()
    aspath_list: (
        Annotated[
            "NetBoxBGPASPathListFilter", strawberry.lazy("netbox_bgp.graphql.filters")
        ]
        | None
    ) = strawberry_django.filter_field()
    aspath_list_id: ID | None = strawberry_django.filter_field()  


@strawberry_django.filter_type(PrefixList, lookups=True)
class NetBoxBGPPrefixListFilter(NetBoxModelFilterMixin):
    name: FilterLookup[str] | None = strawberry_django.filter_field()
    description: FilterLookup[str] | None = strawberry_django.filter_field()
    family: (
        Annotated[
            "NetBoxBGPIPAddressFamilyEnum", strawberry.lazy("netbox_bgp.graphql.enums")
        ]
        | None
    ) = strawberry_django.filter_field()


@strawberry_django.filter_type(PrefixListRule, lookups=True)
class NetBoxBGPPrefixListRuleFilter(NetBoxModelFilterMixin):
    action: (
        Annotated[
            "NetBoxBGPActionEnum", strawberry.lazy("netbox_bgp.graphql.enums")
        ]
        | None
    ) = strawberry_django.filter_field()
    prefix_list: (
        Annotated[
            "NetBoxBGPPrefixListFilter", strawberry.lazy("netbox_bgp.graphql.filters")
        ]
        | None
    ) = strawberry_django.filter_field()
    prefix_list_id: ID | None = strawberry_django.filter_field()



@strawberry_django.filter_type(CommunityList, lookups=True)
class NetBoxBGPCommunityListFilter(NetBoxModelFilterMixin):
    name: FilterLookup[str] | None = strawberry_django.filter_field()
    description: FilterLookup[str] | None = strawberry_django.filter_field()


@strawberry_django.filter_type(CommunityListRule, lookups=True)
class NetBoxBGPCommunityListRuleFilter(NetBoxModelFilterMixin):
    action: (
        Annotated[
            "NetBoxBGPActionEnum", strawberry.lazy("netbox_bgp.graphql.enums")
        ]
        | None
    ) = strawberry_django.filter_field()

    community_list: (
        Annotated[
            "NetBoxBGPCommunityListFilter", strawberry.lazy("netbox_bgp.graphql.filters")
        ]
        | None
    ) = strawberry_django.filter_field()
    community_list_id: ID | None = strawberry_django.filter_field()






from rest_framework.serializers import HyperlinkedIdentityField, ValidationError
from rest_framework.relations import PrimaryKeyRelatedField
from netbox.api.fields import ChoiceField, SerializedPKRelatedField
from netbox.api.serializers import NetBoxModelSerializer
from ipam.api.serializers import IPAddressSerializer, ASNSerializer, PrefixSerializer
from tenancy.api.serializers import TenantSerializer
from dcim.api.serializers import SiteSerializer, DeviceSerializer
from ipam.api.field_serializers import IPNetworkField
from virtualization.api.serializers import VirtualMachineSerializer

from netbox_bgp.models import (
    BGPSession,
    RoutingPolicy,
    BGPPeerGroup,
    Community,
    RoutingPolicyRule,
    PrefixList,
    PrefixListRule,
    CommunityList,
    CommunityListRule,
    ASPathList,
    ASPathListRule
)

from netbox_bgp.choices import CommunityStatusChoices, SessionStatusChoices


class ASPathListSerializer(NetBoxModelSerializer):
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_bgp-api:aspathlist-detail")

    class Meta:
        model = ASPathList
        fields = [
            "id",
            "url",
            "name",
            "display",
            "description",
            "tags",
            "custom_fields",
            "comments",
        ]
        brief_fields = ("id", "url", "display", "name", "description")    


class ASPathListRuleSerializer(NetBoxModelSerializer):
    aspath_list = ASPathListSerializer(nested=True)

    class Meta:
        model = ASPathListRule
        fields = [
            "id",
            "description",
            "tags",
            "custom_fields",
            "display",
            "aspath_list",
            "created",
            "last_updated",
            "index",
            "action",
            "pattern",
            "comments",
        ]
        brief_fields = ("id", "display", "description")


class RoutingPolicySerializer(NetBoxModelSerializer):
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_bgp-api:routingpolicy-detail")

    class Meta:
        model = RoutingPolicy
        fields = (
            "id",
            "url",
            "display",
            "name",
            "description",
            "tags",
            "custom_fields",
            "comments",
        )
        brief_fields = ("id", "url", "display", "name", "description")


class PrefixListSerializer(NetBoxModelSerializer):
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_bgp-api:prefixlist-detail")

    class Meta:
        model = PrefixList
        fields = (
            "id",
            "url",
            "name",
            "display",
            "description",
            "family",
            "tags",
            "custom_fields",
            "comments",
        )
        brief_fields = ("id", "url", "display", "name", "description")


class BGPPeerGroupSerializer(NetBoxModelSerializer):
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_bgp-api:bgppeergroup-detail")

    import_policies = SerializedPKRelatedField(
        queryset=RoutingPolicy.objects.all(),
        serializer=RoutingPolicySerializer,
        nested=True,
        required=False,
        allow_null=True,
        many=True,
    )
    export_policies = SerializedPKRelatedField(
        queryset=RoutingPolicy.objects.all(),
        serializer=RoutingPolicySerializer,
        nested=True,
        required=False,
        allow_null=True,
        many=True,
    )

    class Meta:
        model = BGPPeerGroup
        fields = (
            "id",
            "url",
            "display",
            "name",
            "description",
            "import_policies",
            "export_policies",
            "comments",
            "custom_fields",
        )
        brief_fields = ("id", "url", "display", "name", "description")


class BGPSessionSerializer(NetBoxModelSerializer):
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_bgp-api:bgpsession-detail")
    status = ChoiceField(choices=SessionStatusChoices, required=False)
    site = SiteSerializer(nested=True, required=False, allow_null=True)
    tenant = TenantSerializer(nested=True, required=False, allow_null=True)
    device = DeviceSerializer(nested=True, required=False, allow_null=True)
    virtualmachine = VirtualMachineSerializer(nested=True, required=False, allow_null=True)
    local_address = IPAddressSerializer(nested=True, required=True, allow_null=False)
    remote_address = IPAddressSerializer(nested=True, required=True, allow_null=False)
    local_as = ASNSerializer(nested=True, required=True, allow_null=False)
    remote_as = ASNSerializer(nested=True, required=True, allow_null=False)
    peer_group = BGPPeerGroupSerializer(nested=True, required=False, allow_null=True)
    prefix_list_in = PrefixListSerializer(nested=True, required=False, allow_null=True)
    prefix_list_out = PrefixListSerializer(nested=True, required=False, allow_null=True)
    import_policies = SerializedPKRelatedField(
        queryset=RoutingPolicy.objects.all(),
        serializer=RoutingPolicySerializer,
        nested=True,
        required=False,
        allow_null=True,
        many=True,
    )
    export_policies = SerializedPKRelatedField(
        queryset=RoutingPolicy.objects.all(),
        serializer=RoutingPolicySerializer,
        nested=True,
        required=False,
        allow_null=True,
        many=True,
    )

    class Meta:
        model = BGPSession
        fields = (
            "id",
            "url",
            "tags",
            "custom_fields",
            "display",
            "status",
            "site",
            "tenant",
            "device",
            "virtualmachine",
            "local_address",
            "remote_address",
            "local_as",
            "remote_as",
            "peer_group",
            "import_policies",
            "export_policies",
            "prefix_list_in",
            "prefix_list_out",
            "created",
            "last_updated",
            "name",
            "description",
            "comments",
        )
        brief_fields = ("id", "url", "display", "name", "description")

    def to_representation(self, instance):
        ret = super().to_representation(instance)

        if instance is not None:
            if instance.peer_group:
                for pol in instance.peer_group.import_policies.difference(
                    instance.import_policies.all()
                ):
                    ret["import_policies"].append(
                        RoutingPolicySerializer(
                            pol,
                            context={"request": self.context["request"]},
                            nested=True,
                        ).data
                    )
                for pol in instance.peer_group.export_policies.difference(
                    instance.export_policies.all()
                ):
                    ret["export_policies"].append(
                        RoutingPolicySerializer(
                            pol,
                            context={"request": self.context["request"]},
                            nested=True,
                        ).data
                    )
        return ret


class CommunitySerializer(NetBoxModelSerializer):
    status = ChoiceField(choices=CommunityStatusChoices, required=False)
    tenant = TenantSerializer(nested=True, required=False, allow_null=True)
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_bgp-api:community-detail")

    class Meta:
        model = Community
        fields = (
            "id",
            "url",
            "tags",
            "custom_fields",
            "display",
            "status",
            "tenant",
            "created",
            "last_updated",
            "description",
            "value",
            "site",
            "role",
            "comments",
        )
        brief_fields = ("id", "url", "display", "value", "description")


class CommunityListSerializer(NetBoxModelSerializer):
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_bgp-api:communitylist-detail")

    class Meta:
        model = CommunityList
        fields = (
            "id",
            "url",
            "name",
            "display",
            "description",
            "tags",
            "custom_fields",
            "comments",
        )
        brief_fields = ("id", "url", "display", "name", "description")


class CommunityListRuleSerializer(NetBoxModelSerializer):
    community_list = CommunityListSerializer(nested=True)
    community = CommunitySerializer(nested=True, required=False, allow_null=True)

    class Meta:
        model = CommunityListRule
        fields = (
            "id",
            "tags",
            "custom_fields",
            "display",
            "description",
            "community_list",
            "created",
            "last_updated",
            "action",
            "community",
            "comments",
        )
        brief_fields = ("id", "display", "description")


class RoutingPolicyRuleSerializer(NetBoxModelSerializer):
    match_ip_address = SerializedPKRelatedField(
        queryset=PrefixList.objects.all(),
        serializer=PrefixListSerializer,
        nested=True,
        required=False,
        allow_null=True,
        many=True,
    )
    match_ipv6_address = SerializedPKRelatedField(
        queryset=PrefixList.objects.all(),
        serializer=PrefixListSerializer,
        nested=True,
        required=False,
        allow_null=True,
        many=True,
    )
    routing_policy = RoutingPolicySerializer(nested=True)

    match_community = SerializedPKRelatedField(
        queryset=Community.objects.all(),
        serializer=CommunitySerializer,
        nested=True,
        required=False,
        allow_null=True,
        many=True,
    )
    match_community_list = SerializedPKRelatedField(
        queryset=CommunityList.objects.all(),
        serializer=CommunityListSerializer,
        nested=True,
        required=False,
        allow_null=True,
        many=True,
    )
    match_aspath_list = SerializedPKRelatedField(
        queryset=ASPathList.objects.all(),
        serializer=ASPathListSerializer,
        nested=True,
        required=False,
        allow_null=True,
        many=True,
    )  

    class Meta:
        model = RoutingPolicyRule
        fields = (
            "id",
            "index",
            "display",
            "action",
            "match_ip_address",
            "routing_policy",
            "match_community",
            "match_community_list",
            "match_aspath_list",
            "match_custom",
            "set_actions",
            "match_ipv6_address",
            "description",
            "continue_entry",
            "tags",
            "custom_fields",
            "comments",
        )
        brief_fields = ("id", "display", "description")


class PrefixListRuleSerializer(NetBoxModelSerializer):
    prefix_list = PrefixListSerializer(nested=True)
    prefix = PrefixSerializer(nested=True, required=False, allow_null=True)
    prefix_custom = IPNetworkField(required=False, allow_null=True)

    class Meta:
        model = PrefixListRule
        fields = (
            "id",
            "description",
            "tags",
            "custom_fields",
            "display",
            "prefix_list",
            "created",
            "last_updated",
            "index",
            "action",
            "prefix_custom",
            "ge",
            "le",
            "prefix",
            "comments",
        )
        brief_fields = ("id", "display", "description")




# Generated by Django 5.2.1 on 2025-05-19 23:35

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0207_remove_redundant_indexes'),
        ('ipam', '0081_remove_service_device_virtual_machine_add_parent_gfk_index'),
        ('netbox_bgp', '0032_netbox_bgp'),
        ('virtualization', '0048_populate_mac_addresses'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='bgpsession',
            unique_together={('device', 'local_address', 'local_as', 'remote_address', 'remote_as')},
        ),
        migrations.AddField(
            model_name='bgpsession',
            name='virtualmachine',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='virtualization.virtualmachine'),
        ),
        migrations.AlterUniqueTogether(
            name='bgpsession',
            unique_together={('device', 'local_address', 'local_as', 'remote_address', 'remote_as'), ('virtualmachine', 'local_address', 'local_as', 'remote_address', 'remote_as')},
        ),
    ]



import strawberry

from netbox_bgp.choices import (
    CommunityStatusChoices,
    SessionStatusChoices,
    ActionChoices,
    IPAddressFamilyChoices,
)

__all__ = (
    "NetBoxBGPCommunityStatusEnum",
    "NetBoxBGPSessionStatusEnum",
    "NetBoxBGPActionEnum",
    "NetBoxBGPIPAddressFamilyEnum",
)

NetBoxBGPCommunityStatusEnum = strawberry.enum(CommunityStatusChoices.as_enum(), name="NetBoxBGPCommunityStatusEnum"        )
NetBoxBGPSessionStatusEnum = strawberry.enum(SessionStatusChoices.as_enum(), name="NetBoxBGPSessionStatusEnum")
NetBoxBGPActionEnum = strawberry.enum(ActionChoices.as_enum(), name="NetBoxBGPActionEnum")
NetBoxBGPIPAddressFamilyEnum = strawberry.enum(IPAddressFamilyChoices.as_enum(), name="NetBoxBGPIPAddressFamilyEnum")




from django.conf import settings

from netbox.plugins import PluginMenuButton, PluginMenuItem, PluginMenu


_menu_items_primary = (
    PluginMenuItem(
        link='plugins:netbox_bgp:community_list',
        link_text='Communities',
        permissions=['netbox_bgp.view_community'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:community_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_community'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:community_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_community'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_bgp:communitylist_list',
        link_text='Community Lists',
        permissions=['netbox_bgp.view_communitylist'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:communitylist_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_communitylist'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:communitylist_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_communitylist'],
            ),
        ),
    ),    
    PluginMenuItem(
        link='plugins:netbox_bgp:bgpsession_list',
        link_text='Sessions',
        permissions=['netbox_bgp.view_bgpsession'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:bgpsession_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_bgpsession'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:bgpsession_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_bgpsession'],
            )
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_bgp:routingpolicy_list',
        link_text='Routing Policies',
        permissions=['netbox_bgp.view_routingpolicy'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:routingpolicy_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_routingpolicy'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:routingpolicy_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_routingpolicy'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_bgp:routingpolicyrule_list',
        link_text='Routing Policy Rules',
        permissions=['netbox_bgp.view_routingpolicyrule'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:routingpolicyrule_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_routingpolicyrule'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:routingpolicyrule_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_routingpolicyrule'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_bgp:prefixlist_list',
        link_text='Prefix Lists',
        permissions=['netbox_bgp.view_prefixlist'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:prefixlist_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_prefixlist'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:prefixlist_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_prefixlist'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_bgp:prefixlistrule_list',
        link_text='Prefix List Rules',
        permissions=['netbox_bgp.view_prefixlistrule'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:prefixlistrule_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_prefixlistrule'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:prefixlistrule_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_prefixlistrule'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_bgp:aspathlist_list',
        link_text='AS Path Lists',
        permissions=['netbox_bgp.view_aspathlist'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:aspathlist_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_aspathlist'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:aspathlist_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_aspathlist'],
            ),
        ),
    ),    
    PluginMenuItem(
        link='plugins:netbox_bgp:aspathlistrule_list',
        link_text='AS Path List Rules',
        permissions=['netbox_bgp.view_aspathlistrule'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:aspathlistrule_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_aspathlistrule'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:aspathlistrule_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_aspathlistrule'],
            ),
        ),
    ),    
    PluginMenuItem(
        link='plugins:netbox_bgp:bgppeergroup_list',
        link_text='Peer Groups',
        permissions=['netbox_bgp.view_bgppeergroup'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:bgppeergroup_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_bgppeergroup'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:bgppeergroup_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_bgppeergroup'],
            ),
        ),
    )
)


_menu_items_grouped = (
    PluginMenuItem(
        link='plugins:netbox_bgp:community_list',
        link_text='Communities',
        permissions=['netbox_bgp.view_community'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:community_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_community'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:community_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_community'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_bgp:communitylist_list',
        link_text='Community Lists',
        permissions=['netbox_bgp.view_communitylist'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:communitylist_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_communitylist'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:communitylist_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_communitylist'],
            ),
        ),
    ),    
    PluginMenuItem(
        link='plugins:netbox_bgp:bgpsession_list',
        link_text='Sessions',
        permissions=['netbox_bgp.view_bgpsession'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:bgpsession_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_bgpsession'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:bgpsession_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_bgpsession'],
            )
        ),
    ),

    PluginMenuItem(
        link='plugins:netbox_bgp:bgppeergroup_list',
        link_text='Peer Groups',
        permissions=['netbox_bgp.view_bgppeergroup'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:bgppeergroup_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_bgppeergroup'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:bgppeergroup_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_bgppeergroup'],
            ),
        ),
    )
)

_aspath_list_menu = (
    PluginMenuItem(
        link='plugins:netbox_bgp:aspathlist_list',
        link_text='AS Path Lists',
        permissions=['netbox_bgp.view_aspathlist'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:aspathlist_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_aspathlist'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:aspathlist_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_aspathlist'],
            ),
        ),
    ),  
    PluginMenuItem(
        link='plugins:netbox_bgp:aspathlistrule_list',
        link_text='AS Path List Rules',
        permissions=['netbox_bgp.view_aspathlistrule'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:aspathlistrule_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_aspathlistrule'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:aspathlistrule_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_aspathlistrule'],
            ),
        ),
    ),   
)   

_routing_policy_menu = (
    PluginMenuItem(
        link='plugins:netbox_bgp:routingpolicy_list',
        link_text='Routing Policies',
        permissions=['netbox_bgp.view_routingpolicy'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:routingpolicy_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_routingpolicy'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:routingpolicy_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_routingpolicy'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_bgp:routingpolicyrule_list',
        link_text='Routing Policy Rules',
        permissions=['netbox_bgp.view_routingpolicyrule'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:routingpolicyrule_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_routingpolicyrule'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:routingpolicyrule_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_routingpolicyrule'],
            ),
        ),
    )
)


_prefix_list_menu = (
    PluginMenuItem(
        link='plugins:netbox_bgp:prefixlist_list',
        link_text='Prefix Lists',
        permissions=['netbox_bgp.view_prefixlist'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:prefixlist_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_prefixlist'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:prefixlist_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_prefixlist'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_bgp:prefixlistrule_list',
        link_text='Prefix List Rules',
        permissions=['netbox_bgp.view_prefixlistrule'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_bgp:prefixlistrule_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_bgp.add_prefixlistrule'],
            ),
            PluginMenuButton(
                link='plugins:netbox_bgp:prefixlistrule_bulk_import',
                title='Import',
                icon_class='mdi mdi-upload',
                permissions=['netbox_bgp.add_prefixlistrule'],
            ),
        ),
    ),
)

plugin_settings = settings.PLUGINS_CONFIG.get('netbox_bgp', {})

if plugin_settings.get('top_level_menu'):
    menu = PluginMenu(  
        label="BGP",
        groups=(
            ("BGP", _menu_items_grouped),
            ("Prefix Lists", _prefix_list_menu),
            ("Routing Policies", _routing_policy_menu),
            ("AS Path Lists", _aspath_list_menu),
        ),
        icon_class="mdi mdi-bootstrap",
    )
else:
    menu_items = _menu_items_primary



from netbox.api.viewsets import NetBoxModelViewSet
from rest_framework.routers import APIRootView

from .serializers import (
    BGPSessionSerializer, RoutingPolicySerializer, BGPPeerGroupSerializer,
    CommunitySerializer, PrefixListSerializer, PrefixListRuleSerializer,
    RoutingPolicyRuleSerializer, CommunityListSerializer, CommunityListRuleSerializer,
    ASPathListSerializer, ASPathListRuleSerializer
)
from netbox_bgp.models import (
    BGPSession, RoutingPolicy, BGPPeerGroup,
    Community, PrefixList, PrefixListRule,
    RoutingPolicyRule, CommunityList, CommunityListRule,
    ASPathList, ASPathListRule
)
from netbox_bgp.filtersets import (
    BGPSessionFilterSet, RoutingPolicyFilterSet, BGPPeerGroupFilterSet,
    CommunityFilterSet, PrefixListFilterSet, PrefixListRuleFilterSet,
    RoutingPolicyRuleFilterSet, CommunityListFilterSet, CommunityListRuleFilterSet,
    ASPathListFilterSet, ASPathListRuleFilterSet
)

class RootView(APIRootView):
    def get_view_name(self):
        return 'BGP'
    

class BGPSessionViewSet(NetBoxModelViewSet):
    queryset = BGPSession.objects.all()
    serializer_class = BGPSessionSerializer
    filterset_class = BGPSessionFilterSet


class RoutingPolicyViewSet(NetBoxModelViewSet):
    queryset = RoutingPolicy.objects.all()
    serializer_class = RoutingPolicySerializer
    filterset_class = RoutingPolicyFilterSet


class RoutingPolicyRuleViewSet(NetBoxModelViewSet):
    queryset = RoutingPolicyRule.objects.all()
    serializer_class = RoutingPolicyRuleSerializer
    filterset_class = RoutingPolicyRuleFilterSet


class BGPPeerGroupViewSet(NetBoxModelViewSet):
    queryset = BGPPeerGroup.objects.all()
    serializer_class = BGPPeerGroupSerializer
    filterset_class = BGPPeerGroupFilterSet


class CommunityViewSet(NetBoxModelViewSet):
    queryset = Community.objects.all()
    serializer_class = CommunitySerializer
    filterset_class = CommunityFilterSet


class CommunityListViewSet(NetBoxModelViewSet):
    queryset = CommunityList.objects.all()
    serializer_class = CommunityListSerializer
    filterset_class = CommunityListFilterSet


class CommunityListRuleViewSet(NetBoxModelViewSet):
    queryset = CommunityListRule.objects.all()
    serializer_class = CommunityListRuleSerializer
    filterset_class = CommunityListRuleFilterSet


class PrefixListViewSet(NetBoxModelViewSet):
    queryset = PrefixList.objects.all()
    serializer_class = PrefixListSerializer
    filterset_class = PrefixListFilterSet


class PrefixListRuleViewSet(NetBoxModelViewSet):
    queryset = PrefixListRule.objects.all()
    serializer_class = PrefixListRuleSerializer
    filterset_class = PrefixListRuleFilterSet


class ASPathListViewSet(NetBoxModelViewSet):
    queryset = ASPathList.objects.all()
    serializer_class = ASPathListSerializer
    filterset_class = ASPathListFilterSet


class ASPathListRuleViewSet(NetBoxModelViewSet):
    queryset = ASPathListRule.objects.all()
    serializer_class = ASPathListRuleSerializer
    filterset_class = ASPathListRuleFilterSet



# Generated by Django 4.2.7 on 2023-12-14 09:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0029_netbox_bgp'),
    ]

    operations = [
        migrations.AddField(
            model_name='bgppeergroup',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgpsession',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='community',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='prefixlist',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='prefixlistrule',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='prefixlistrule',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='routingpolicy',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='routingpolicyrule',
            name='comments',
            field=models.TextField(blank=True),
        ),
    ]



# Generated by Django 4.1.9 on 2023-05-16 06:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_bgp', '0027_netbox_bgp'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bgpsession',
            name='device',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='dcim.device'),
        ),
    ]
